# -*- coding: utf-8 -*-
# Copyright (C) 2016-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for ban lists"""

from django.contrib.auth.models import User
from django.urls import reverse

from allauth.account.models import EmailAddress
from bs4 import BeautifulSoup

from postorius.tests.utils import ViewTestCase


class ListBansTest(ViewTestCase):

    def setUp(self):
        super().setUp()
        # Create domain `example.com` in Mailman
        self.domain = self.mm_client.create_domain('example.com')
        self.m_list = self.domain.create_list('test_list')
        self.test_user = User.objects.create_user(
            'test_user', 'test_user@example.com', 'pwd')
        self.test_superuser = User.objects.create_superuser(
            'test_superuser', 'test_superuser@example.com', 'pwd')
        for user in (self.test_user, self.test_superuser):
            EmailAddress.objects.create(
                user=user, email=user.email, verified=True)
        self.client.login(username="test_superuser", password='pwd')
        self.url = reverse('list_bans', args=['test_list.example.com'])
        self.bans_list = self.m_list.bans

    def test_login_redirect_for_anonymous(self):
        self.client.logout()
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 302)

    def test_no_access_for_basic_user(self):
        self.client.logout()
        self.client.login(username="test_user", password='pwd')
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)

    def test_access_for_superuser(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)

    def test_context_contains_create_form(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue('addban_form' in response.context)
        soup = BeautifulSoup(response.content, "html.parser")
        tag_input = soup.find("input", {
            "class": "form-control", "id": "id_email",
            "name": "email", "type": "text",
            })
        self.assertIsNotNone(tag_input)
        tag_button = soup.find("button", {
            "class": "btn btn-primary", "name": "add", "type": "submit",
            })
        self.assertIsNotNone(tag_button)

    def test_context_contains_delete_forms(self):
        banned = ['banned{}@example.com'.format(i) for i in range(1, 10)]
        for ban in banned:
            self.bans_list.add(ban)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content, "html.parser")
        for ban in banned:
            self.assertIsNotNone(soup.find("input", {
                "name": "email", "type": "hidden", "value": ban,
                }))
        self.assertEqual(len(soup.find_all("button", {
            "class": "btn btn-danger btn-xs",
            "name": "del", "type": "submit",
            })), 9)

    def test_add_ban(self):
        response = self.client.post(self.url, {
            'email': 'banned@example.com',
            'add': True,
            })
        self.assertRedirects(response, self.url)
        self.assertIn('banned@example.com', self.bans_list)
        self.assertHasSuccessMessage(response)

        self.bans_list.remove('banned@example.com')

    def test_del_ban(self):
        self.bans_list.add('banned@example.com')
        self.assertIn('banned@example.com', self.bans_list)
        response = self.client.post(self.url, {
            'email': 'banned@example.com',
            'del': True,
            })
        self.assertRedirects(response, self.url)
        self.assertNotIn('banned@example.com', self.bans_list)
        self.assertHasSuccessMessage(response)

    def test_del_unknown_ban(self):
        self.assertNotIn('banned@example.com', self.bans_list)
        response = self.client.post(self.url, {
            'email': 'banned@example.com',
            'del': True,
            })
        self.assertRedirects(response, self.url)
        message = self.assertHasErrorMessage(response)
        self.assertIn('is not banned', message)

    def test_add_ban_duplicate(self):
        self.bans_list.add('banned@example.com')
        self.assertIn('banned@example.com', self.bans_list)
        response = self.client.post(self.url, {
            'email': 'banned@example.com',
            'add': True,
            })
        self.assertRedirects(response, self.url)
        message = self.assertHasErrorMessage(response)
        self.assertIn('is already banned', message)

        self.bans_list.remove('banned@example.com')


class GlobalBansTest(ListBansTest):

    def setUp(self):
        super().setUp()
        self.url = reverse('global_bans')
        self.bans_list = self.mm_client.bans



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#


from django import forms
from django.utils.translation import gettext_lazy as _

from postorius.forms.fields import NullBooleanRadioSelect
from postorius.utils import LANGUAGES, with_empty_choice


class UserPreferences(forms.Form):

    """
    Form handling the user's global, address and subscription based preferences
    """

    def __init__(self, *args, **kwargs):
        self._preferences = kwargs.pop('preferences', None)
        super(UserPreferences, self).__init__(*args, **kwargs)

    @property
    def initial(self):
        # Redirect to the preferences, this allows setting the preferences
        # after instanciation and it will also set the initial data.
        return self._preferences or {}

    @initial.setter
    def initial(self, value):
        pass

    choices = ((True, _('Yes')), (False, _('No')))

    delivery_mode_choices = (("regular", _('Regular')),
                             ("plaintext_digests", _('Plain Text Digests')),
                             ("mime_digests", _('Mime Digests')),
                             ("summary_digests", _('Summary Digests')))
    delivery_status_choices = (
        ("enabled", _('Enabled')), ("by_user", _('Disabled')))
    delivery_status = forms.ChoiceField(
        widget=forms.RadioSelect,
        choices=delivery_status_choices,
        required=False,
        label=_('Delivery status'),
        help_text=_(
            'Set this option to Enabled to receive messages posted to this '
            'mailing list. Set it to Disabled if you want to stay subscribed, '
            'but don\'t want mail delivered to you for a while (e.g. you\'re '
            'going on vacation). If you disable mail delivery, don\'t forget '
            'to re-enable it when you come back; it will not be automatically '
            're-enabled.'))
    delivery_mode = forms.ChoiceField(
        widget=forms.Select(),
        choices=with_empty_choice(delivery_mode_choices),
        required=False,
        label=_('Delivery mode'),
        help_text=_(
            'If you select summary digests , you\'ll get posts bundled '
            'together (usually one per day but possibly more on busy lists), '
            'instead of singly when they\'re sent. Your mail reader may or '
            'may not support MIME digests. In general MIME digests are '
            'preferred, but if you have a problem reading them, select '
            'plain text digests.'))
    receive_own_postings = forms.NullBooleanField(
        widget=NullBooleanRadioSelect(choices=choices),
        required=False,
        label=_('Receive own postings'),
        help_text=_(
            'Ordinarily, you will get a copy of every message you post to the '
            'list. If you don\'t want to receive this copy, set this option '
            'to No.'
        ))
    acknowledge_posts = forms.NullBooleanField(
        widget=NullBooleanRadioSelect(choices=choices),
        required=False,
        label=_('Acknowledge posts'),
        help_text=_(
            'Receive acknowledgement mail when you send mail to the list?'))
    hide_address = forms.NullBooleanField(
        widget=NullBooleanRadioSelect(choices=choices),
        required=False,
        label=_('Hide address'),
        help_text=_(
            'When someone views the list membership, your email address is '
            'normally shown (in an obscured fashion to thwart spam '
            'harvesters). '
            'If you do not want your email address to show up on this '
            'membership roster at all, select Yes for this option.'))
    receive_list_copy = forms.NullBooleanField(
        widget=NullBooleanRadioSelect(choices=choices),
        required=False,
        label=_('Receive list copies (possible duplicates)'),
        help_text=_(
            'When you are listed explicitly in the To: or Cc: headers of a '
            'list message, you can opt to not receive another copy from the '
            'mailing list. Select Yes to receive copies. '
            'Select No to avoid receiving copies from the mailing list'))

    preferred_language = forms.ChoiceField(
        widget=forms.Select(),
        choices=with_empty_choice(LANGUAGES),
        required=False,
        label=_('Preferred language'),
        help_text=_(
            'Preferred language for your interactions with Mailman. When '
            'this is set, it will override the MailingList\'s preferred '
            'language. This affects which language is used for your '
            'email notifications and such.'))

    class Meta:

        """
        Class to define the name of the fieldsets and what should be
        included in each.
        """
        layout = [["User Preferences", "acknowledge_posts", "hide_address",
                   "receive_list_copy", "receive_own_postings",
                   "delivery_mode", "delivery_status", "preferred_language"]]

    def save(self):
        if not self.changed_data:
            return
        for key in self.changed_data:
            if self.cleaned_data[key] is not None:
                # None: nothing set yet. Remember to remove this test
                # when Mailman accepts None as a "reset to default"
                # value.
                self._preferences[key] = self.cleaned_data[key]
        self._preferences.save()


class UserPreferencesFormset(forms.BaseFormSet):

    def __init__(self, *args, **kwargs):
        self._preferences = kwargs.pop('preferences')
        kwargs["initial"] = self._preferences
        super(UserPreferencesFormset, self).__init__(*args, **kwargs)

    def _construct_form(self, i, **kwargs):
        form = super(UserPreferencesFormset, self)._construct_form(i, **kwargs)
        form._preferences = self._preferences[i]
        return form

    def save(self):
        for form in self.forms:
            form.save()



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

from django.test import TestCase

from postorius.forms.user_forms import UserPreferences


class UserPreferencesTest(TestCase):

    def test_form_fields_valid(self):
        form = UserPreferences({
            'acknowledge_posts': 'True',
            'hide_address': 'True',
            'receive_list_copy': 'False',
            'receive_own_postings': 'False',
        })
        self.assertTrue(form.is_valid())



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

from postorius.forms.domain_forms import *  # noqa
from postorius.forms.fields import *  # noqa
from postorius.forms.list_forms import *  # noqa
from postorius.forms.member_forms import *  # noqa
from postorius.forms.system import *  # noqa
from postorius.forms.user_forms import *  # noqa



from django.core.exceptions import ValidationError

import pytest

from postorius.forms.validators import validate_uuid_or_email


class TestValidators:

    def test_validate_uuid_or_email(self):
        # Test valid email.
        emails = ['aperson@example.com', 'user@localhost', 'bp@localhost.com']
        for email in emails:
            assert validate_uuid_or_email(email) == email
        # Test invalid email uuid.
        uuids = ['00000000000000000000000000000034',
                 '00000000000000000000000000000084']
        for uuid in uuids:
            assert validate_uuid_or_email(uuid) == uuid

        # Test invalid email.
        invalid_emails = [
            'missingdomain@', '@missinglocal', 'mis2']
        for email in invalid_emails:
            with pytest.raises(ValidationError):
                validate_uuid_or_email(email)
        # Test invalid uuid.
        invalid_uuids = ['02394', '0923402340000000023']
        for uuid in invalid_uuids:
            with pytest.raises(ValidationError):
                validate_uuid_or_email(uuid)



# -*- coding: utf-8 -*-
# Copyright (C) 2016-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django.contrib.auth.models import User
from django.test.utils import override_settings
from django.urls import reverse

from allauth.account.models import EmailAddress
from django_mailman3.lib.mailman import get_mailman_user

from postorius.forms import ChangeSubscriptionForm, UserPreferences
from postorius.models import Mailman404Error, MailmanUser
from postorius.tests.utils import ViewTestCase


class MailmanUserTest(ViewTestCase):
    """
    Tests for the mailman user preferences settings page.
    """

    def setUp(self):
        super(MailmanUserTest, self).setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.foo_list = self.domain.create_list('foo')
        self.foo_list.send_welcome_message = False
        self.user = User.objects.create_user(
            'user', 'user@example.com', 'testpass')
        EmailAddress.objects.create(
            user=self.user, email=self.user.email, verified=True, primary=True)
        self.mm_user = MailmanUser.objects.create_from_django(self.user)

    def test_address_preferences_not_logged_in(self):
        self.assertRedirectsToLogin(reverse('user_address_preferences'))

    def test_subscriptions_not_logged_in(self):
        self.assertRedirectsToLogin(reverse('ps_user_profile'))

    def test_subscriptions_logged_in(self):
        self.client.login(username='user', password='testpass')
        response = self.client.get(reverse('ps_user_profile'))
        self.assertEqual(response.status_code, 200)

    def test_address_based_preferences(self):
        self.client.login(username='user', password='testpass')
        self.mm_user.add_address('user2@example.com')
        self.mm_user.add_address('user3@example.com')
        response = self.client.get(reverse('user_address_preferences'))
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context["formset"]), 3)

    # this test needs re-thinking with the way we've hacked preferences
    # it has been disabled for now
    # def test_preferences_none(self):
    #     # Mailman does not accept None values for boolean preferences. When
    #     # those preferences are unset, they must be excluded from the POST
    #     # data.
    #     self.client.login(username='user', password='testpass')
    #     self.foo_list.subscribe(self.user.email, pre_verified=True,
    #                             pre_confirmed=True, pre_approved=True)
    #     prefs_with_none = (
    #         'receive_own_postings', 'acknowledge_posts',
    #         'hide_address', 'receive_list_copy',
    #         )
    #     # Prepare a Preferences subclass that will check the POST data
    #     import mailmanclient._client

    #     class TestingPrefs(mailmanclient._client.Preferences):
    #         testcase = self

    #         def save(self):
    #             for pref in prefs_with_none:
    #                 self.testcase.assertNotIn(pref, self._changed_rest_data)
    #     # Now check the relevant URLs
    #     with patch('mailmanclient._client.Preferences') as pref_class:
    #         pref_class.side_effect = TestingPrefs
    #         # Simple forms
    #         for url in (
    #                 reverse('user_mailmansettings'),
    #                 reverse('user_list_options',
    #                     args=[self.foo_list.list_id]),
    #                 ):
    #             response = self.client.post(
    #                 url, dict((pref, None) for pref in prefs_with_none))
    #             self.assertEqual(response.status_code, 302)
    #         # Formsets
    #         for url in ('user_address_preferences',
    #                     'user_subscription_preferences'):
    #             url = reverse(url)
    #             post_data = dict(
    #                 ('form-0-%s' % pref, None)
    #                 for pref in prefs_with_none)
    #             post_data.update({
    #                 'form-TOTAL_FORMS': '1',
    #                 'form-INITIAL_FORMS': '0',
    #                 'form-MAX_NUM_FORMS': ''
    #             })
    #             response = self.client.post(url, post_data)
    #             self.assertEqual(response.status_code, 302)

    @override_settings(AUTOCREATE_MAILMAN_USER=False)
    def test_subscriptions_no_mailman_user(self):
        # Existing Django users without a corresponding Mailman user must not
        # cause views to crash.
        user = User.objects.create_user(
            'old-user', 'old-user@example.com', 'testpass')
        EmailAddress.objects.create(
            user=user, email=user.email, verified=True)
        self.client.login(username='old-user', password='testpass')
        self.assertRaises(Mailman404Error, MailmanUser.objects.get,
                          address=user.email)
        response = self.client.get(reverse('ps_user_profile'))
        self.assertEqual(response.status_code, 200)
        # The Mailman user must have been created
        self.assertIsNotNone(MailmanUser.objects.get(address=user.email))

    def test_presence_of_form_in_user_global_settings(self):
        self.client.login(username='user', password='testpass')
        response = self.client.get(reverse('user_mailmansettings'))
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.context['form'], UserPreferences)

    def test_presence_of_form_in_user_subscription_preferences(self):
        self.client.login(username='user', password='testpass')
        self.foo_list.subscribe(self.user.email, pre_verified=True,
                                pre_confirmed=True, pre_approved=True)
        response = self.client.get(reverse('user_subscription_preferences'))
        self.assertEqual(response.status_code, 200)
        self.assertIsNotNone(response.context['formset'])
        self.assertEqual(len(response.context['formset']), 1)

    def test_presence_of_form_in_user_list_options(self):
        self.client.login(username='user', password='testpass')
        self.foo_list.subscribe(self.user.email, pre_verified=True,
                                pre_confirmed=True, pre_approved=True)
        response = self.client.get(reverse('user_list_options',
                                           args=[self.foo_list.list_id]))
        self.assertEqual(response.status_code, 200)
        self.assertIsInstance(response.context['form'],
                              UserPreferences)
        self.assertIsInstance(response.context['change_subscription_form'],
                              ChangeSubscriptionForm)

    def test_list_options_shows_all_addresses(self):
        self.client.login(username='user', password='testpass')
        self.foo_list.subscribe(self.user.email, pre_verified=True,
                                pre_confirmed=True, pre_approved=True)
        # Add another email
        EmailAddress.objects.create(
            user=self.user, email='anotheremail@example.com', verified=True)
        user = self.mm_client.get_user('user@example.com')
        address = user.add_address('anotheremail@example.com')
        address.verify()
        # Check response
        response = self.client.get(reverse('user_list_options',
                                           args=[self.foo_list.list_id]))
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'anotheremail@example.com')

    def _set_primary(self, user, mm_user):
        for addr in mm_user.addresses:
            addr.verify()
        mm_user.preferred_address = user.email

    def test_change_subscription_to_new_email(self):
        # Test that we can change subscription to a new email.
        self.client.login(username='user', password='testpass')
        user = self.mm_client.get_user('user@example.com')
        EmailAddress.objects.create(
            user=self.user, email='anotheremail@example.com', verified=True)
        address = user.add_address('anotheremail@example.com')
        address.verify()
        self.foo_list.subscribe(
            self.user.email,
            pre_verified=True, pre_confirmed=True, pre_approved=True)
        # Now, first verify that the list_options page has all the emails.
        # Check response
        response = self.client.get(reverse('user_list_options',
                                           args=[self.foo_list.list_id]))
        self.assertContains(response, 'anotheremail@example.com')
        self.assertContains(
            response,
            '<option value="user@example.com"'
            ' selected>user@example.com</option>')
        member = self.mm_client.get_member(
            self.foo_list.list_id, 'user@example.com')
        self.assertIsNotNone(member)
        # Initially, all preferences are none. Let's set it to something
        # custom.
        self.assertIsNone(member.preferences.get('acknowledge_posts'))
        member.preferences['acknowledge_posts'] = True
        member.preferences.save()
        # now, let's switch the subscription to a new user.
        response = self.client.post(
            reverse('change_subscription', args=(self.foo_list.list_id, )),
            {'subscriber': 'anotheremail@example.com'}
        )
        self.assertEqual(response.status_code, 302)
        self.assertHasSuccessMessage(response)
        member_new = self.mm_client.get_member(
            self.foo_list.list_id, 'anotheremail@example.com')
        self.assertIsNotNone(member_new)
        # There is no 'member_id' attribute, so we simply use the self_link to
        # compare and make sure that the Member object is same.
        self.assertEqual(member.self_link, member_new.self_link)
        self.assertEqual(member_new.subscription_mode, 'as_address')
        # Also, assert that the new member's preferences are same.
        self.assertEqual(member.preferences['acknowledge_posts'],
                         member_new.preferences['acknowledge_posts'])

    def test_change_subscription_to_from_primary_address(self):
        # Test that we can change subscription to a new email.
        self.client.login(username='user', password='testpass')
        user = self.mm_client.get_user('user@example.com')
        self._set_primary(self.user, user)
        self.foo_list.subscribe(
            self.user.email,
            pre_verified=True, pre_confirmed=True, pre_approved=True)
        # Now, first verify that the list_options page has the primary address.
        response = self.client.get(reverse('user_list_options',
                                           args=[self.foo_list.list_id]))
        self.assertContains(response, 'Primary Address (user@example.com)')
        self.assertContains(
            response,
            '<option value="user@example.com" '
            'selected>user@example.com</option>')
        member = self.mm_client.get_member(
            self.foo_list.list_id, 'user@example.com')
        self.assertIsNotNone(member)
        # Initially, all preferences are none. Let's set it to something
        # custom.
        self.assertIsNone(member.preferences.get('acknowledge_posts'))
        member.preferences['acknowledge_posts'] = True
        member.preferences.save()
        # now, let's switch the subscription to a new user.
        response = self.client.post(
            reverse('change_subscription', args=(self.foo_list.list_id, )),
            {'subscriber': str(user.user_id)}
        )
        self.assertEqual(response.status_code, 302)
        self.assertHasSuccessMessage(response)
        new_member = self.mm_client.get_member(
            self.foo_list.list_id, 'user@example.com')
        self.assertIsNotNone(new_member)
        self.assertEqual(new_member.subscription_mode, 'as_user')
        # we can't compare against the preferences object of `member` since the
        # resource is now Deleted due to unsubscribe-subscribe dance.
        self.assertEqual(new_member.preferences['acknowledge_posts'], True)

    def test_already_subscribed(self):
        self.client.login(username='user', password='testpass')

        self.foo_list.subscribe(
            self.user.email,
            pre_verified=True, pre_confirmed=True, pre_approved=True)
        # Now, first verify that the list_options page has all the emails.
        # Check response
        response = self.client.get(reverse('user_list_options',
                                           args=[self.foo_list.list_id]))
        self.assertContains(
            response,
            '<option value="user@example.com" '
            'selected>user@example.com</option>')
        # now, let's switch the subscription to a new user.
        response = self.client.post(
            reverse('change_subscription', args=(self.foo_list.list_id, )),
            {'subscriber': 'user@example.com'}
        )
        self.assertEqual(response.status_code, 302)
        error = self.assertHasErrorMessage(response)
        self.assertIn('You are already subscribed', error)

    def test_already_subscribed_with_primary_address(self):
        # Test that we can change subscription to a new email.
        self.client.login(username='user', password='testpass')
        user = self.mm_client.get_user('user@example.com')
        self._set_primary(self.user, user)
        self.foo_list.subscribe(
            user.user_id,
            pre_verified=True, pre_confirmed=True, pre_approved=True)
        # Now, first verify that the list_options page has the primary address.
        response = self.client.get(reverse('user_list_options',
                                           args=[self.foo_list.list_id]))
        self.assertContains(
            response,
            ('<option value="{}" selected>Primary Address (user@example.com)'
             '</option>').format(user.user_id))
        # now, let's switch the subscription to a new user.
        response = self.client.post(
            reverse('change_subscription', args=(self.foo_list.list_id, )),
            {'subscriber': str(user.user_id)}
        )
        self.assertEqual(response.status_code, 302)
        error = self.assertHasErrorMessage(response)
        self.assertIn('You are already subscribed', error)

    def test_list_options_sets_preferred_address(self):
        # Test that preferred address is set.
        mm_user = get_mailman_user(self.user)
        self.assertIsNone(mm_user.preferred_address)
        self.foo_list.subscribe(
            self.user.email,
            pre_verified=True, pre_confirmed=True, pre_approved=True)
        self.client.login(username='user', password='testpass')
        self.client.get(reverse('user_list_options',
                                args=[self.foo_list.list_id]))
        self.assertEqual(mm_user.preferred_address.email, self.user.email)



# -*- coding: utf-8 -*-
# Copyright (C) 2019-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

import datetime
from unittest.mock import patch

from django.test import SimpleTestCase, override_settings

from postorius.templatetags.date_helpers import datetime_parse
from postorius.templatetags.p_gravatar import gravatar


class TestDatetimeParser(SimpleTestCase):

    def test_datetime_parse(self):
        value = '2005-08-01T07:49:23'
        parsed_date = datetime_parse(value)
        # Make sure the parsing actually worked.
        self.assertFalse(parsed_date is value)
        self.assertTrue(isinstance(parsed_date, datetime.datetime))
        self.assertEqual(parsed_date.strftime('%Y %m %d, %H %M'),
                         '2005 08 01, 07 49')

    def test_invalid_dateimtime(self):
        # Things that can't be parsed as datetime are returned as None.
        value = 'someraindomstring'
        parsed_date = datetime_parse(value)
        self.assertTrue(parsed_date is None)
        # Things that can be parsed as datetime, but are invalid dates return
        # the value back.
        # 32nd Aug for example:
        value = '2005-08-32T07:49:23'
        parsed_date = datetime_parse(value)
        self.assertTrue(parsed_date is value)


class TestGravatar(SimpleTestCase):

    def test_gravatar(self):
        """Test that we call gravatar library."""
        with patch('postorius''.templatetags.'
                   'p_gravatar.gravatar_orig') as mock_grav:
            gravatar('aperson@example.com')
            self.assertTrue(mock_grav.called)
            mock_grav.assert_called_with('aperson@example.com')
        html = gravatar('bperson@example.com')
        self.assertEqual(
            html,
            '<img class="gravatar" src="https://secure.gravatar.com/avatar/'
            'a100672ae026b5b7a7fb2929ff533e1e.jpg?s=80&amp;d=mm&amp;r=g" '
            'width="80" height="80" alt="" />')

    @override_settings(HYPERKITTY_ENABLE_GRAVATAR=False)
    def test_disabled_gravatar(self):
        with patch('postorius''.templatetags.'
                   'p_gravatar.gravatar_orig') as mock_grav:
            resp = gravatar('aperson@example.com')
            self.assertFalse(mock_grav.called)
            self.assertEqual(resp, '')



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#
import json
import time
from email import message_from_bytes, message_from_file

from django.contrib.auth.models import User

from allauth.account.models import EmailAddress

from postorius.tests.utils import ViewTestCase, get_test_file, reverse
from postorius.views.rest import get_attachments, parse


class TestRestViews(ViewTestCase):

    def setUp(self):
        super().setUp()
        self.user = User.objects.create(
            email='bob@example.com', password='testpass')
        self.su = User.objects.create_superuser(
            username='alice', email='alice@example.com', password='testpass')
        EmailAddress.objects.create(
            user=self.user, email=self.user.email, verified=True)
        EmailAddress.objects.create(
            user=self.su, email=self.su.email, verified=True)
        self.domain = self.mm_client.create_domain('example.com')
        self.foo_list = self.domain.create_list('foo')

    def test_get_attachments(self):
        with open(get_test_file('test-email-attachment.txt')) as fp:
            attachments = get_attachments(fp.read())
        self.assertEqual(len(attachments), 1)
        at1 = attachments[0]
        self.assertEqual(at1[1], 'signature.asc')
        self.assertEqual(at1[2], 'application/pgp-signature')

    def test_get_attachments_bad_charset(self):
        # While this part is supposed to be handled by Django-Mailman3.
        # This should nor fail or error out.
        with open(get_test_file('bad-email.txt')) as fp:
            attachments = get_attachments(fp.read())
        self.assertEqual(len(attachments), 0)

    def test_parsed_email(self):
        with open(get_test_file('simple-email.txt')) as fp:
            retval = parse(fp.read())
        self.assertTrue('This is a test message.' in retval['body'])
        for header_str in ['From: gil <puntogil@libero.it>',
                           'Content-Type: multipart/mixed;',
                           'Sender: devel-bounces@lists.fedoraproject.org']:
            self.assertTrue(header_str, retval['headers'])


class TestGetHeldMessage(ViewTestCase):

    def setUp(self):
        super().setUp()
        self.user = User.objects.create(
            email='bob@example.com', password='testpass')
        self.su = User.objects.create_superuser(
            username='alice', email='alice@example.com', password='testpass')
        EmailAddress.objects.create(
            user=self.user, email=self.user.email, verified=True)
        EmailAddress.objects.create(
            user=self.su, email=self.su.email, verified=True)
        self.domain = self.mm_client.create_domain('example.com')
        self.foo_list = self.domain.create_list('foo')
        self.inq = self.mm_client.queues['in']
        with open(get_test_file('test-email-attachment.txt')) as fp:
            message = message_from_file(fp)
        # Now, we inject a non-member post to this email list to simulate a
        # held message.
        self.inq.inject('foo.example.com', message.as_string())
        # We sleep till this message is processed and held.
        time.sleep(2)

    def test_get_held_message(self):
        self.client.login(username='alice', password='testpass')
        response = self.client.get(reverse('rest_held_message',
                                           args=['foo.example.com', '1']))
        self.assertEqual(response.status_code, 200)
        content = json.loads(response.content.decode('utf-8'))
        self.assertEqual(
            sorted(list(content.keys())),
            ['attachments', 'hold_date', 'msg', 'msgid', 'reason', 'sender', 'subject'])   # noqa
        self.assertEqual(content['msgid'], 1)
        self.assertEqual(len(content['attachments']), 1)
        attachment_uri = content['attachments'][0]
        # Now we will get this attachment.
        self.client.get(attachment_uri)

    def test_get_raw_held_message(self):
        self.client.login(username='alice', password='testpass')
        response = self.client.get(reverse('rest_held_message',
                                           args=['foo.example.com', '1']),
                                   {'raw': 'True'})
        self.assertEqual(response.status_code, 200)
        email = message_from_bytes(response.content)
        self.assertIsNotNone(email)

    def test_get_attachments_for_held_message(self):
        self.client.login(username='alice', password='testpass')
        response = self.client.get(reverse('rest_held_message',
                                           args=['foo.example.com', '1']))
        self.assertEqual(response.status_code, 200)
        attachment_uri = json.loads(response.content.decode('utf-8'))['attachments'][0][0]   # noqa: E501
        # Now we will get this attachment.
        response = self.client.get(attachment_uri)
        self.assertEqual(response.status_code, 200)
        self.assertTrue('BEGIN PGP SIGNATURE' in str(response.content))



# -*- coding: utf-8 -*-
# Copyright (C) 2016-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.urls import reverse

from allauth.account.models import EmailAddress
from django_mailman3.models import MailDomain

from postorius.tests.utils import ViewTestCase


class DomainDeleteTest(ViewTestCase):
    """Tests for the domain delete page."""

    def setUp(self):
        super(DomainDeleteTest, self).setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.foo_list = self.domain.create_list('foo')

        self.user = User.objects.create_user(
            'testuser', 'test@example.com', 'testpass')
        self.superuser = User.objects.create_superuser(
            'testsu', 'su@example.com', 'testpass')
        self.owner = User.objects.create_user(
            'testowner', 'owner@example.com', 'testpass')
        self.moderator = User.objects.create_user(
            'testmoderator', 'moderator@example.com', 'testpass')
        for user in (self.user, self.superuser, self.owner, self.moderator):
            EmailAddress.objects.create(
                user=user, email=user.email, verified=True)
        self.foo_list.add_owner('owner@example.com')
        self.foo_list.add_moderator('moderator@example.com')
        MailDomain.objects.create(
            site=Site.objects.get_current(), mail_domain='example.com')
        self.url = reverse('domain_delete', args=['example.com'])

    def test_access_anonymous(self):
        # Anonymous users users can't delete domains
        self.assertRedirectsToLogin(self.url)

    def test_access_basic_user(self):
        # Basic users can't delete domains
        self.client.login(username='testuser', password='testpass')
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)

    def test_access_moderators(self):
        # Moderators can't delete domains
        self.client.login(username='testmoderator', password='testpass')
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)

    def test_access_owners(self):
        # Owners can't delete domains
        self.client.login(username='testowner', password='testpass')
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)

    def test_domain_delete_confirm(self):
        # The user should be ask to confirm domain deletion on GET requests
        self.client.login(username='testsu', password='testpass')
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(self.mm_client.domains), 1)
        self.assertEqual(len(self.mm_client.lists), 1)

    def test_domain_delete_page_lists_all_mailinglists(self):
        # Test that the domain delete page lists all the mailing lists
        # associated with the domain.
        self.client.login(username='testsu', password='testpass')
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(
            'This would delete 1 lists, some of which are'
            in str(response.content))
        self.assertTrue('foo@example.com' in str(response.content))

    def test_domain_delete(self):
        # The domain should be deleted
        self.client.login(username='testsu', password='testpass')
        response = self.client.post(self.url)
        self.assertRedirects(response, reverse('domain_index'))
        self.assertEqual(len(self.mm_client.domains), 0)
        self.assertEqual(len(self.mm_client.lists), 0)
        self.assertHasSuccessMessage(response)
        self.assertFalse(
            MailDomain.objects.filter(mail_domain='example.com').exists())



# -*- coding: utf-8 -*-
# Copyright (C) 2016-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for list settings"""

from django.contrib.auth.models import User
from django.urls import reverse

from allauth.account.models import EmailAddress

from postorius.models import List
from postorius.tests.utils import ViewTestCase
from postorius.views.list import SETTINGS_FORMS


class ListSettingsTest(ViewTestCase):
    """
    Tests for the list settings page.
    """

    def setUp(self):
        super(ListSettingsTest, self).setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.foo_list = self.domain.create_list('foo')
        self.user = User.objects.create_user(
            'testuser', 'test@example.com', 'testpass')
        self.superuser = User.objects.create_superuser(
            'testsu', 'su@example.com', 'testpass')
        self.owner = User.objects.create_user(
            'testowner', 'owner@example.com', 'testpass')
        self.moderator = User.objects.create_user(
            'testmoderator', 'moderator@example.com', 'testpass')
        for user in (self.user, self.superuser, self.owner, self.moderator):
            EmailAddress.objects.create(
                user=user, email=user.email, verified=True)
        self.foo_list.add_owner('owner@example.com')
        self.foo_list.add_moderator('moderator@example.com')

    def tearDown(self):
        self.domain.delete()

    def test_page_not_accessible_if_not_logged_in(self):
        for section_name in SETTINGS_FORMS:
            url = reverse('list_settings', args=('foo.example.com',
                                                 section_name))
            self.assertRedirectsToLogin(url)

    def test_page_not_accessible_for_unprivileged_users(self):
        self.client.login(username='testuser', password='testpass')
        for section_name in SETTINGS_FORMS:
            url = reverse('list_settings', args=('foo.example.com',
                                                 section_name))
            response = self.client.get(url)
            self.assertEqual(response.status_code, 403)

    def test_not_accessible_for_moderator(self):
        self.client.login(username='testmoderator', password='testpass')
        for section_name in SETTINGS_FORMS:
            url = reverse('list_settings', args=('foo.example.com',
                                                 section_name))
            response = self.client.get(url)
            self.assertEqual(response.status_code, 403)

    def test_page_accessible_for_owner(self):
        self.client.login(username='testowner', password='testpass')
        for section_name in SETTINGS_FORMS:
            url = reverse('list_settings', args=('foo.example.com',
                                                 section_name))
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)

    def test_page_accessible_for_superuser(self):
        self.client.login(username='testsu', password='testpass')
        for section_name in SETTINGS_FORMS:
            url = reverse('list_settings', args=('foo@example.com',
                                                 section_name))
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)

    def test_archiving_policy(self):
        self.assertEqual(self.foo_list.settings['archive_policy'], 'public')
        self.client.login(username='testsu', password='testpass')
        url = reverse('list_settings', args=('foo.example.com', 'archiving'))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.context["form"].initial['archive_policy'], 'public')
        response = self.client.post(url, {'archive_policy': 'private'})
        self.assertRedirects(response, url)
        self.assertHasSuccessMessage(response)
        # Get a new list object to avoid caching
        m_list = List.objects.get(fqdn_listname='foo.example.com')
        self.assertEqual(m_list.settings['archive_policy'], 'private')

    def test_archivers(self):
        self.assertEqual(dict(self.foo_list.archivers),
                         {'mhonarc': True, 'prototype': True,
                          'mail-archive': True})
        self.client.login(username='testsu', password='testpass')
        url = reverse('list_settings', args=('foo.example.com', 'archiving'))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context["form"].initial['archivers'],
                         ['mail-archive', 'mhonarc', 'prototype'])
        response = self.client.post(
            url, {'archive_policy': 'public', 'archivers': ['prototype']})
        self.assertRedirects(response, url)
        self.assertHasSuccessMessage(response)
        # Get a new list object to avoid caching
        m_list = List.objects.get(fqdn_listname='foo.example.com')
        self.assertEqual(dict(m_list.archivers),
                         {'mhonarc': False, 'prototype': True,
                          'mail-archive': False})

    def test_bug_117(self):
        self.assertEqual(self.foo_list.settings['first_strip_reply_to'], False)
        self.client.login(username='testsu', password='testpass')
        url = reverse(
            'list_settings', args=('foo.example.com', 'alter_messages'))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        form = response.context["form"]
        self.assertEqual(
            form.initial['first_strip_reply_to'], False)
        post_data = dict(
            (key, self.foo_list.settings[key])
            for key in form.fields)
        post_data['first_strip_reply_to'] = True
        response = self.client.post(url, post_data)
        self.assertRedirects(response, url)
        self.assertHasSuccessMessage(response)
        # Get a new list object to avoid caching
        m_list = List.objects.get(fqdn_listname='foo.example.com')
        self.assertEqual(m_list.settings['first_strip_reply_to'], True)

    def test_list_identity_allow_empty_prefix_and_desc(self):
        self.assertEqual(self.foo_list.settings['subject_prefix'], '[Foo] ')
        self.assertEqual(self.foo_list.settings['description'], '')
        self.client.login(username='testsu', password='testpass')
        url = reverse('list_settings',
                      args=('foo.example.com', 'list_identity'))
        response = self.client.post(url, {
            'subject_prefix': '',
            'description': '',
            'advertised': 'True',
            'preferred_language': 'en',
            'member_roster_visibility': 'public',
            'newsgroup_moderation': 'none',
            })
        self.assertRedirects(response, url)
        self.assertHasSuccessMessage(response)
        # Get a new list object to avoid caching
        m_list = List.objects.get(fqdn_listname='foo.example.com')
        self.assertEqual(m_list.settings['subject_prefix'], '')
        self.assertEqual(m_list.settings['description'], '')

    def test_respond_to_post_requests(self):
        self.assertTrue(self.foo_list.settings['respond_to_post_requests'])
        self.client.login(username='testsu', password='testpass')
        url = reverse('list_settings',
                      args=('foo.example.com', 'automatic_responses'))
        response = self.client.post(
            url,
            {'respond_to_post_requests': False,
             'autorespond_owner': 'none',
             'autorespond_postings': 'none',
             'autorespond_requests': 'none',
             'send_welcome_message': True,
             'send_goodbye_message': True,
             'autoresponse_grace_period': '20d'})
        self.assertRedirects(response, url)
        self.assertHasSuccessMessage(response)
        mlist = List.objects.get(fqdn_listname='foo.example.com')
        self.assertFalse(mlist.settings['respond_to_post_requests'])

    def test_list_subscription_requests(self):
        self.client.login(username='testowner', password='testpass')
        url = reverse('list_subscription_requests', args=('foo.example.com',))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        # Since there are no pending subscription requests, this page should be
        # empty.
        self.assertContains(response, '<small>(0)</small>')
        self.assertTrue(
            b'There are currently no subscription requests for this list.'
            in response.content)
        self.assertNotContains(response, '<div class="paginator">')
        # We set subscription policy to 'confirm', which should wait for the
        # user approval.
        self.foo_list.settings['subscription_policy'] = 'confirm'
        self.foo_list.settings.save()
        self.foo_list.subscribe('someone@example.com', pre_verified=True)
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        # Check that the request is not shown in pending subscription requests.
        self.assertTrue('someone@example.com' not in str(response.content))
        # Now we set the subscription policy to moderate so that all
        # subscriptions are held for moderator approval.
        self.foo_list.settings['subscription_policy'] = 'moderate'
        self.foo_list.settings.save()
        self.foo_list.subscribe('test@example.com', pre_verified=True)
        self.foo_list.subscribe('owner@example.com', pre_verified=True)
        self.foo_list.subscribe('moderator@example.com', pre_verified=True)
        # Now there should be three subscription requests pending.
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        for email in ('test@example.com', 'owner@example.com',
                      'moderator@example.com'):
            self.assertTrue(email in str(response.content))
        self.assertContains(response, '<small>(3)</small>')
        # Verify that the list is paginated
        self.assertContains(response, '<div class="paginator">')

    def test_handle_subscription_request(self):
        self.client.login(username='testowner', password='testpass')
        self.foo_list.settings['subscription_policy'] = 'moderate'
        self.foo_list.settings.save()
        self.foo_list.subscribe('test@example.com', pre_verified=True)
        # There should be one pending subscription request.
        self.assertEqual(len(self.foo_list.requests), 1)
        token = self.foo_list.requests[0]['token']
        url = reverse('handle_subscription_request',
                      args=('foo.example.com', token, 'accept'))
        response = self.client.get(url)
        # On success, user is redirected to list_subscription_requests page.
        self.assertTrue(response.status_code, 302)
        self.assertTrue(response.url,
                        '/postorius/lists/foo.example.com/subscription_requests')  # noqa
        self.assertEqual(len(self.foo_list.requests), 0)

    def test_remove_all_subscribers(self):
        self.client.login(username='testowner', password='testpass')
        # Let's add 10 subscribers to the list.
        for each in range(10):
            self.foo_list.subscribe('test-{}@example.com'.format(each),
                                    pre_verified=True,
                                    pre_approved=True,
                                    pre_confirmed=True)
        self.assertEqual(len(self.foo_list.members), 10)
        # First lets see the correct form is rendered when we GET
        url = reverse('unsubscribe_all', args=('foo.example.com',))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(b'Confirm Removal of All Members' in response.content)
        # If we POST to the above url, we will unsubscribe all the users.
        self.assertEqual(len(self.foo_list.members), 10)
        # Now, let's remove all the subscribers
        self.client.post(url)
        self.assertEqual(len(self.foo_list.members), 0)

    def test_message_acceptance(self):
        initial_values = {
            'acceptable_aliases': [],
            'require_explicit_destination': True,
            'administrivia': True,
            'default_member_action': 'defer',
            'default_nonmember_action': 'hold',
            'max_message_size': 40,
            'max_num_recipients': 10,
        }
        self.client.login(username='testsu', password='testpass')
        url = reverse('list_settings',
                      args=('foo.example.com', 'message_acceptance'))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        for field, value in initial_values.items():
            self.assertEqual(
                self.foo_list.settings[field], value,
                'Field: {}'.format(field))
            self.assertEqual(
                response.context["form"].initial[field], value,
                'Field: {}'.format(field))
        updated_values = {
            'acceptable_aliases': ['bar@example.com'],
            'require_explicit_destination': False,
            'administrivia': False,
            'default_member_action': 'accept',
            'default_nonmember_action': 'accept',
            'max_message_size': 100,
            'max_num_recipients': 100,
        }
        response = self.client.post(url, updated_values)
        self.assertRedirects(response, url)
        self.assertHasSuccessMessage(response)
        # Get a new list object to avoid caching
        m_list = List.objects.get(fqdn_listname='foo.example.com')
        for field, value in updated_values.items():
            self.assertEqual(
                m_list.settings[field], value,
                'Field: {}'.format(field))

    def test_mass_subscribe_options(self):
        self.client.login(username='testsu', password='testpass')
        mlist = List.objects.get(fqdn_listname='foo.example.com')
        updated_values = {
            'emails': 'john3@example.com',
            'pre_verified': True,
            'pre_confirmed': True,
            'pre_approved': True,
        }
        url = reverse('mass_subscribe',
                      args=('foo.example.com',))
        response = self.client.post(url, updated_values)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(mlist.members), 1)

        # Now, let's see if we can moderate.
        self.foo_list.settings['subscription_policy'] = 'moderate'
        self.foo_list.settings.save()

        updated_values = {
            'emails': 'john@example.com',
            'pre_confirmed': True,
            'pre_approved': True,
            'pre_verified': True
        }

        self.assertEqual(len(mlist.members), 1)
        response = self.client.post(url, updated_values)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(mlist.members), 2)
        # With pre_confirmed = False, there should be a pending subscription
        # event.
        updated_values = {
            'emails': 'john2@example.com',
            'pre_confirmed': True,
            'pre_verified': True
        }
        response = self.client.post(url, updated_values)
        self.assertEqual(response.status_code, 200)
        # Now, there should be a pending request to confirm.
        self.assertEqual(len(mlist.members), 2)
        self.assertEqual(len(mlist.requests), 1)
        self.assertEqual(mlist.requests[0].get('token_owner'), 'moderator')

        # Test mass subscribe without all options.
        self.foo_list.settings['subscription_policy'] = 'confirm_then_moderate'
        self.foo_list.settings.save()

        updated_values = {
            'emails': 'john4@example.com',
            'pre_confirmed': False,
            'pre_verified': True
        }

        response = self.client.post(url, updated_values)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(mlist.requests), 2)
        self.assertEqual(mlist.requests[1].get('token_owner'), 'subscriber')

        # Test mass subscribe an invitation.  Subscription policy doesn't
        # matter, so just leave it as it was.
        updated_values = {
            'emails': 'john5@example.com',
            'invitation': True
        }
        response = self.client.post(url, updated_values)
        self.assertEqual(response.status_code, 200)
        # And now there are 3 requests.
        self.assertEqual(len(mlist.requests), 3)
        self.assertEqual(mlist.requests[2].get('token_owner'), 'subscriber')

    def test_set_list_of_strings_field_empty(self):
        self.client.login(username='testsu', password='testpass')
        m_list = List.objects.get(fqdn_listname='foo.example.com')
        settings = m_list.settings
        settings['accept_these_nonmembers'] = ['^bar*@example.[com|org]']
        settings.save()

        url = reverse('list_settings',
                      args=('foo.example.com', 'message_acceptance'))
        updated_values = {
            'accept_these_nonmembers': [],
            'require_explicit_destination': True,
            'administrivia': True,
            'default_member_action': 'defer',
            'default_nonmember_action': 'hold',
            'max_message_size': 40,
            'max_num_recipients': 10,
        }
        response = self.client.post(url, updated_values)
        self.assertEqual(response.status_code, 302)
        self.assertHasSuccessMessage(response)
        # Get a new list object to avoid caching
        settings._reset_cache()
        self.assertEqual(settings.get('accept_these_nonmembers'), [])



# -*- coding: utf-8 -*-
# Copyright (C) 2018-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

from django.urls import reverse

from postorius.tests.utils import ViewTestCase


class AnonymousSubscribeTest(ViewTestCase):
    """Tests for anonymous subscribe page."""

    def setUp(self):
        super().setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.foo_list = self.domain.create_list('foo')

    def test_anonymous_subscribe(self):
        url = reverse('list_anonymous_subscribe', args=('foo.example.com',))
        response = self.client.post(url, dict(email='aperson@example.com'))
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url,
                         reverse('list_summary', args=('foo.example.com',)))
        # 302 responses have no context, hence we can't check the actual
        # success message.
        response = self.client.post(url,
                                    dict(email='bperson@example.com'),
                                    follow=True)
        success_msg = list(response.context.get('messages'))[0]
        self.assertEqual(success_msg.tags, 'success')
        self.assertEqual(success_msg.message,
                         'Please check your inbox for further instructions')
        # Make sure that there are two pending requests in Mailing List.
        self.assertEqual(len(self.foo_list.requests), 2)
        for req in self.foo_list.requests:
            self.assertEqual(req['token_owner'], 'subscriber')
            self.assertEqual(req['list_id'], 'foo.example.com')

    def test_anonymous_subscribe_get(self):
        # Issue 185
        # Test that anonymous subscribe with a GET doesn't raise a 500 error.
        url = reverse('list_anonymous_subscribe', args=('foo.example.com',))
        response = self.client.get(url)
        self.assertEqual(response.status_code, 405)



# -*- coding: utf-8 -*-
# Copyright (C) 2016-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for list header matches"""


from django.contrib.auth.models import User
from django.urls import reverse

from allauth.account.models import EmailAddress
from bs4 import BeautifulSoup

from postorius.tests.utils import ViewTestCase


class ListHeaderMatchesTest(ViewTestCase):
    """
    Tests for the list settings page.
    """

    def setUp(self):
        super(ListHeaderMatchesTest, self).setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.mlist = self.domain.create_list('list')
        self.user = User.objects.create_user(
            'testuser', 'test@example.com', 'testpass')
        self.superuser = User.objects.create_superuser(
            'testsu', 'su@example.com', 'testpass')
        self.owner = User.objects.create_user(
            'testowner', 'owner@example.com', 'testpass')
        self.moderator = User.objects.create_user(
            'testmoderator', 'moderator@example.com', 'testpass')
        for user in (self.user, self.superuser, self.owner, self.moderator):
            EmailAddress.objects.create(
                user=user, email=user.email, verified=True)
        self.mlist.add_owner('owner@example.com')
        self.mlist.add_moderator('moderator@example.com')

    def test_page_not_accessible_if_not_logged_in(self):
        url = reverse('list_header_matches', args=['list.example.com'])
        self.assertRedirectsToLogin(url)

    def test_page_not_accessible_for_unprivileged_users(self):
        self.client.login(username='testuser', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)

    def test_not_accessible_for_moderator(self):
        self.client.login(username='testmoderator', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)

    def test_page_accessible_for_owner(self):
        self.client.login(username='testowner', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_page_accessible_for_superuser(self):
        self.client.login(username='testsu', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_show_existing(self):
        self.mlist.header_matches.add(
            header='testheader', pattern='testpattern', action='discard')
        self.client.login(username='testowner', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(len(response.context["formset"]), 2)
        self.assertEqual(
            response.context["formset"].initial,
            [{'header': 'testheader', 'pattern': 'testpattern',
              'action': 'discard'}])
        self.assertContains(response, 'testheader')
        self.assertContains(response, 'testpattern')
        soup = BeautifulSoup(response.content, "html.parser")
        tag_form = soup.find("select", {"name": "form-0-action"})
        self.assertIsNotNone(tag_form)
        tag_option = tag_form.find("option", value="discard", selected=True)
        self.assertIsNotNone(tag_option)
        # the new header match subform should not have ORDER or DELETE fields
        self.assertNotContains(response, 'form-1-ORDER')
        self.assertNotContains(response, 'form-1-DELETE')

    def test_add(self):
        self.client.login(username='testowner', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        data = {
            # Management form
            'form-TOTAL_FORMS': '1',
            'form-INITIAL_FORMS': '0',
            'form-MIN_NUM_FORMS': '0',
            'form-MAX_NUM_FORMS': '1000',
            # New form
            'form-0-header': 'testheader',
            'form-0-pattern': 'testpattern',
            'form-0-action': 'discard',
        }
        response = self.client.post(url, data)
        self.assertRedirects(response, url)
        self.assertHasSuccessMessage(response)
        self.assertEqual(len(self.mlist.header_matches), 1)
        hm = self.mlist.header_matches[0]
        self.assertEqual(hm.header, 'testheader')
        self.assertEqual(hm.pattern, 'testpattern')
        self.assertEqual(hm.action, 'discard')

    def test_add_empty(self):
        self.client.login(username='testowner', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        data = {
            # Management form
            'form-TOTAL_FORMS': '1',
            'form-INITIAL_FORMS': '0',
            'form-MIN_NUM_FORMS': '0',
            'form-MAX_NUM_FORMS': '1000',
            # New form
            'form-0-header': '',
            'form-0-pattern': '',
            'form-0-action': '',
        }
        response = self.client.post(url, data)
        self.assertRedirects(response, url)
        self.assertHasNoMessage(response)
        self.assertEqual(len(self.mlist.header_matches), 0)

    def test_add_empty_header(self):
        self.client.login(username='testowner', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        data = {
            # Management form
            'form-TOTAL_FORMS': '1',
            'form-INITIAL_FORMS': '0',
            'form-MIN_NUM_FORMS': '0',
            'form-MAX_NUM_FORMS': '1000',
            # New form
            'form-0-header': '',
            'form-0-pattern': 'testpattern',
            'form-0-action': '',
        }
        response = self.client.post(url, data)
        self.assertEqual(response.status_code, 200)
        self.assertHasNoMessage(response)
        self.assertEqual(
            response.context["formset"].errors,
            [{'header': ['Please enter a header.']}])
        self.assertEqual(len(self.mlist.header_matches), 0)

    def test_add_empty_pattern(self):
        self.client.login(username='testowner', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        data = {
            # Management form
            'form-TOTAL_FORMS': '1',
            'form-INITIAL_FORMS': '0',
            'form-MIN_NUM_FORMS': '0',
            'form-MAX_NUM_FORMS': '1000',
            # New form
            'form-0-header': 'testheader',
            'form-0-pattern': '',
            'form-0-action': '',
        }
        response = self.client.post(url, data)
        self.assertEqual(response.status_code, 200)
        self.assertHasNoMessage(response)
        self.assertEqual(
            response.context["formset"].errors,
            [{'pattern': ['Please enter a pattern.']}])
        self.assertEqual(len(self.mlist.header_matches), 0)

    def test_edit(self):
        self.mlist.header_matches.add(
            header='testheader', pattern='testpattern', action='discard')
        self.client.login(username='testowner', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        data = {
            # Management form
            'form-TOTAL_FORMS': '2',
            'form-INITIAL_FORMS': '1',
            'form-MIN_NUM_FORMS': '0',
            'form-MAX_NUM_FORMS': '1000',
            # Existing pattern
            'form-0-header': 'testheader-changed',
            'form-0-pattern': 'testpattern-changed',
            'form-0-action': 'hold',
            'form-0-ORDER': '1',
            'form-0-DELETE': '',
            # New form
            'form-1-header': '',
            'form-1-pattern': '',
            'form-1-action': '',
        }
        response = self.client.post(url, data)
        self.assertRedirects(response, url)
        self.assertHasSuccessMessage(response)
        self.assertEqual(len(self.mlist.header_matches), 1)
        hm = self.mlist.header_matches[0]
        self.assertEqual(hm.header, 'testheader-changed')
        self.assertEqual(hm.pattern, 'testpattern-changed')
        self.assertEqual(hm.action, 'hold')

    def test_edit_empty(self):
        self.mlist.header_matches.add(
            header='testheader', pattern='testpattern', action='discard')
        self.client.login(username='testowner', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        data = {
            # Management form
            'form-TOTAL_FORMS': '2',
            'form-INITIAL_FORMS': '1',
            'form-MIN_NUM_FORMS': '0',
            'form-MAX_NUM_FORMS': '1000',
            # Existing pattern
            'form-0-header': '',
            'form-0-pattern': '',
            'form-0-action': '',
            'form-0-ORDER': '1',
            'form-0-DELETE': '',
            # New form
            'form-1-header': '',
            'form-1-pattern': '',
            'form-1-action': '',
        }
        response = self.client.post(url, data)
        self.assertEqual(response.status_code, 200)
        self.assertHasNoMessage(response)
        self.assertEqual(
            response.context["formset"].errors,
            [{'header': ['Please enter a header.'],
              'pattern': ['Please enter a pattern.'],
              }, {}])
        self.assertEqual(len(self.mlist.header_matches), 1)
        hm = self.mlist.header_matches[0]
        self.assertEqual(hm.header, 'testheader')
        self.assertEqual(hm.pattern, 'testpattern')
        self.assertEqual(hm.action, 'discard')

    def test_delete(self):
        self.mlist.header_matches.add(
            header='testheader-1', pattern='testpattern-1', action='discard')
        self.mlist.header_matches.add(
            header='testheader-2', pattern='testpattern-2', action='discard')
        self.client.login(username='testowner', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        data = {
            # Management form
            'form-TOTAL_FORMS': '3',
            'form-INITIAL_FORMS': '2',
            'form-MIN_NUM_FORMS': '0',
            'form-MAX_NUM_FORMS': '1000',
            # Existing pattern
            'form-0-header': 'testheader-1',
            'form-0-pattern': 'testpattern-1',
            'form-0-action': 'hold',
            'form-0-ORDER': '1',
            'form-0-DELETE': '1',
            'form-1-header': 'testheader-2',
            'form-1-pattern': 'testpattern-2',
            'form-1-action': 'discard',
            'form-1-ORDER': '2',
            'form-1-DELETE': '',
            # New form
            'form-2-header': '',
            'form-2-pattern': '',
            'form-2-action': '',
        }
        response = self.client.post(url, data)
        self.assertRedirects(response, url)
        self.assertHasSuccessMessage(response)
        self.assertEqual(len(self.mlist.header_matches), 1)
        hm = self.mlist.header_matches[0]
        self.assertEqual(hm.header, 'testheader-2')
        self.assertEqual(hm.pattern, 'testpattern-2')
        self.assertEqual(hm.action, 'discard')

    def test_move_up(self):
        self.mlist.header_matches.add(
            header='testheader-1', pattern='testpattern-1', action='discard')
        self.mlist.header_matches.add(
            header='testheader-2', pattern='testpattern-2', action='discard')
        self.mlist.header_matches.add(
            header='testheader-3', pattern='testpattern-3', action='discard')
        self.client.login(username='testowner', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        data = {
            # Management form
            'form-TOTAL_FORMS': '4',
            'form-INITIAL_FORMS': '3',
            'form-MIN_NUM_FORMS': '0',
            'form-MAX_NUM_FORMS': '1000',
            # Existing patterns
            'form-0-header': 'testheader-1',
            'form-0-pattern': 'testpattern-1',
            'form-0-action': 'discard',
            'form-0-ORDER': '1',
            'form-0-DELETE': '',
            'form-1-header': 'testheader-2',
            'form-1-pattern': 'testpattern-2',
            'form-1-action': 'discard',
            'form-1-ORDER': '3',
            'form-1-DELETE': '',
            'form-2-header': 'testheader-3',
            'form-2-pattern': 'testpattern-3',
            'form-2-action': 'discard',
            'form-2-ORDER': '2',
            'form-2-DELETE': '',
            # New form
            'form-3-header': '',
            'form-3-pattern': '',
            'form-3-action': '',
        }
        response = self.client.post(url, data)
        self.assertRedirects(response, url)
        self.assertHasSuccessMessage(response)
        self.assertEqual(len(self.mlist.header_matches), 3)
        self.assertEqual(
            [(hm.header, hm.pattern, hm.action)
             for hm in self.mlist.header_matches],
            [('testheader-1', 'testpattern-1', 'discard'),
             ('testheader-3', 'testpattern-3', 'discard'),
             ('testheader-2', 'testpattern-2', 'discard')]
            )

    def test_move_down(self):
        self.mlist.header_matches.add(
            header='testheader-1', pattern='testpattern-1', action='discard')
        self.mlist.header_matches.add(
            header='testheader-2', pattern='testpattern-2', action='discard')
        self.mlist.header_matches.add(
            header='testheader-3', pattern='testpattern-3', action='discard')
        self.client.login(username='testowner', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        data = {
            # Management form
            'form-TOTAL_FORMS': '4',
            'form-INITIAL_FORMS': '3',
            'form-MIN_NUM_FORMS': '0',
            'form-MAX_NUM_FORMS': '1000',
            # Existing patterns
            'form-0-header': 'testheader-1',
            'form-0-pattern': 'testpattern-1',
            'form-0-action': 'discard',
            'form-0-ORDER': '2',
            'form-0-DELETE': '',
            'form-1-header': 'testheader-2',
            'form-1-pattern': 'testpattern-2',
            'form-1-action': 'discard',
            'form-1-ORDER': '1',
            'form-1-DELETE': '',
            'form-2-header': 'testheader-3',
            'form-2-pattern': 'testpattern-3',
            'form-2-action': 'discard',
            'form-2-ORDER': '3',
            'form-2-DELETE': '',
            # New form
            'form-3-header': '',
            'form-3-pattern': '',
            'form-3-action': '',
        }
        response = self.client.post(url, data)
        self.assertRedirects(response, url)
        self.assertHasSuccessMessage(response)
        self.assertEqual(len(self.mlist.header_matches), 3)
        self.assertEqual(
            [(hm.header, hm.pattern, hm.action)
             for hm in self.mlist.header_matches],
            [('testheader-2', 'testpattern-2', 'discard'),
             ('testheader-1', 'testpattern-1', 'discard'),
             ('testheader-3', 'testpattern-3', 'discard')]
            )

    def test_same_order(self):
        self.mlist.header_matches.add(
            header='testheader-1', pattern='testpattern-1', action='discard')
        self.mlist.header_matches.add(
            header='testheader-2', pattern='testpattern-2', action='discard')
        self.client.login(username='testowner', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        data = {
            # Management form
            'form-TOTAL_FORMS': '3',
            'form-INITIAL_FORMS': '2',
            'form-MIN_NUM_FORMS': '0',
            'form-MAX_NUM_FORMS': '1000',
            # Existing patterns
            'form-0-header': 'testheader-1',
            'form-0-pattern': 'testpattern-1',
            'form-0-action': 'discard',
            'form-0-ORDER': '1',
            'form-0-DELETE': '',
            'form-1-header': 'testheader-2',
            'form-1-pattern': 'testpattern-2',
            'form-1-action': 'discard',
            'form-1-ORDER': '1',
            'form-1-DELETE': '',
            # New form
            'form-2-header': '',
            'form-2-pattern': '',
            'form-2-action': '',
        }
        response = self.client.post(url, data)
        self.assertEqual(response.status_code, 200)
        self.assertHasNoMessage(response)
        for form_errors in response.context["formset"].errors:
            self.assertEqual(len(form_errors), 0)
        self.assertEqual(
            response.context["formset"].non_form_errors(),
            ['Header matches must have distinct orders.'])
        self.assertEqual(len(self.mlist.header_matches), 2)
        self.assertEqual(
            [(hm.header, hm.pattern, hm.action)
             for hm in self.mlist.header_matches],
            [('testheader-1', 'testpattern-1', 'discard'),
             ('testheader-2', 'testpattern-2', 'discard')]
            )

    def test_add_existing(self):
        self.mlist.header_matches.add(
            header='testheader', pattern='testpattern', action='discard')
        self.client.login(username='testowner', password='testpass')
        url = reverse('list_header_matches', args=['list.example.com'])
        data = {
            # Management form
            'form-TOTAL_FORMS': '2',
            'form-INITIAL_FORMS': '1',
            'form-MIN_NUM_FORMS': '0',
            'form-MAX_NUM_FORMS': '1000',
            # Existing patterns
            'form-0-header': 'testheader',
            'form-0-pattern': 'testpattern',
            'form-0-action': 'discard',
            'form-0-ORDER': '1',
            'form-0-DELETE': '',
            # New form
            'form-1-header': 'testheader',
            'form-1-pattern': 'testpattern',
            'form-1-action': 'hold',
        }
        response = self.client.post(url, data)
        self.assertRedirects(response, url)
        self.assertHasErrorMessage(response)
        self.assertEqual(len(self.mlist.header_matches), 1)
        self.assertEqual(
            [(hm.header, hm.pattern, hm.action)
             for hm in self.mlist.header_matches],
            [('testheader', 'testpattern', 'discard')])



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#
from unittest.mock import patch

from django.contrib.auth.models import User
from django.test import TestCase

from mailmanclient import Client


def server_error(requset):
    raise Exception()


class TestUtils(TestCase):

    def test_404_page(self):
        response = self.client.get('/postorius/not-here', follow=True)
        self.assertEqual(response.status_code, 404)
        self.assertTrue(
            '<h1>Page not found</h1>' in str(response.content))
        self.assertTrue(
            '<div class="alert alert-danger">' in str(response.content))

    @patch.object(Client, 'get_list')
    def test_403_page(self, mock_get_list):
        user = User.objects.create_user(
            'testuser', 'test@example.com', 'testpass')
        self.client.force_login(user)
        response = self.client.get(
            '/postorius/lists/foolist.example.org/settings/', follow=True)
        self.assertEqual(response.status_code, 403)
        self.assertTrue(
            '<h1>Forbidden</h1>' in str(response.content))
        self.assertTrue(
            '<div class="alert alert-danger">' in str(response.content))

    def test_500_page(self):
        su = User.objects.create_superuser(
            'su', 'test@example.com', 'testpass')
        self.client.force_login(su)
        response = self.client.get('/500', follow=True)
        self.assertEqual(response.status_code, 500)
        self.assertTrue(b'<h1>Server error</h1>' in response.content)



# -*- coding: utf-8 -*-
# Copyright (C) 2020-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#


from django.contrib.auth.models import User

from allauth.account.models import EmailAddress

from postorius.auth.utils import user_is_in_list_roster
from postorius.tests.utils import ViewTestCase


class AuthTestCase(ViewTestCase):

    def setUp(self):
        super().setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.mlist = self.domain.create_list('authlist')
        self.user1 = User.objects.create_user(
            'aperson', 'aperson@example.com', 'pass')
        EmailAddress.objects.create(user=self.user1,
                                    email=self.user1.email,
                                    verified=True)
        self.user2 = User.objects.create_user(
            'bperson', 'BPERSON@example.com', 'pass')
        EmailAddress.objects.create(user=self.user2,
                                    email=self.user2.email,
                                    verified=True)
        self.mlist.subscribe(
            'APERSON@example.com',
            pre_verified=True, pre_confirmed=True, pre_approved=True)
        self.mlist.subscribe(
            'bperson@example.com',
            pre_verified=True, pre_confirmed=True, pre_approved=True)

    def test_user_in_roster_case_sensitivity(self):
        # Test that if Core stores the email adddress with Upper case letters
        # and Postorius stores it in lower case, the matching works.
        self.assertTrue(
            user_is_in_list_roster(self.user1, self.mlist, 'members'))

        self.assertTrue(
            user_is_in_list_roster(self.user2, self.mlist, 'members'))



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#
import unittest

from django.test import TestCase

from postorius.forms.list_forms import (
    ArchiveSettingsForm, ChangeSubscriptionForm, DigestSettingsForm,
    DMARCMitigationsForm, ListAnonymousSubscribe, ListAutomaticResponsesForm,
    ListHeaderMatchForm, ListIdentityForm, ListMassRemoval,
    ListMassSubscription, ListNew, ListSubscribe, MemberModeration,
    MemberPolicyForm, MessageAcceptanceForm)
from postorius.forms.system import AddBanForm
from postorius.tests.utils import create_mock_list


class TestListSubscribe(TestCase):

    def test_required_fields_only(self):
        user_emails = ['bob@example.com', 'anne@example.com']
        form = ListSubscribe(user_emails, None, None,
                             dict(subscriber='bob@example.com'))
        self.assertTrue(form.is_valid())

    def test_email_is_only_from_choices(self):
        user_emails = ['bob@example.com', 'anne@example.com']
        form = ListSubscribe(user_emails, None, None,
                             dict(subscriber='alice@example.com',
                                  display_name='Alice'))
        self.assertFalse(form.is_valid())
        self.assertTrue('subscriber' in form.errors)
        self.assertTrue('Select a valid choice.' in
                        form.errors['subscriber'][0])

    def test_subscribe_works(self):
        user_emails = ['someone@example.com']
        form = ListSubscribe(user_emails, None, None,
                             {'subscriber': 'someone@example.com',
                              'display_name': 'Someone'})
        self.assertTrue(form.is_valid())

    def test_subscribe_fails(self):
        user_emails = ['someone@example.com']
        form = ListSubscribe(user_emails, None, None,
                             {'subscriber': 'notaemail',
                              'display_name': 'Someone'})
        self.assertFalse(form.is_valid())
        self.assertTrue('subscriber' in form.errors.keys())
        self.assertEqual(form.errors['subscriber'][0],
                         'Select a valid choice.'
                         ' notaemail is not one of the available choices.')

    def test_subscribe_validates_email(self):
        user_emails = ['something']
        form = ListSubscribe(user_emails, None, None,
                             {'subscriber': 'something',
                              'display_name': 'Someone'})
        self.assertFalse(form.is_valid())
        self.assertTrue('subscriber' in form.errors.keys())
        self.assertEqual(form.errors['subscriber'][0],
                         'Please enter a valid email address.')

    def test_subscribe_with_user_uuid(self):
        user_emails = ['aperson@example.com']
        form = ListSubscribe(user_emails,
                             '00000000000000000000000000000004',
                             'aperson@example.com',
                             {'subscriber': '00000000000000000000000000000004'}
                             )
        self.assertEqual(form.is_valid(), True)

    def test_subscribe_with_invalid_uuid(self):
        # Test that invalid UUID returns errors.
        user_emails = ['aperson@example.com']
        form = ListSubscribe(user_emails,
                             '4',
                             'aperson@example.com',
                             {'subscriber': '4'})
        self.assertEqual(form.is_valid(), False)
        self.assertEqual(form.errors['subscriber'][0],
                         'Please enter a valid email address.')


class TestChangeSubscription(TestCase):

    def test_subscription_changes_only_to_user_addresses(self):
        user_emails = ['one@example.com', 'two@example.com']
        form = ChangeSubscriptionForm(user_emails, None, None,
                                      {'subscriber': 'abcd@d.com'})
        self.assertFalse(form.is_valid())
        self.assertTrue('subscriber' in form.errors.keys())
        self.assertEqual(form.errors['subscriber'][0],
                         'Select a valid choice. '
                         'abcd@d.com is not one of the available choices.')

    def test_subscription_works(self):
        user_emails = ['one@example.com', 'two@example.com']
        form = ChangeSubscriptionForm(user_emails, None, None,
                                      {'subscriber': 'two@example.com'})
        self.assertTrue(form.is_valid())

    def test_subscription_form_labels(self):
        user_emails = ['one@example.com', 'two@example.com']
        form = ChangeSubscriptionForm(user_emails, None, None, {})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.fields['subscriber'].label, 'Select Email')

    def test_form_validity(self):
        form = ChangeSubscriptionForm(
            ['email@example.com', 'john@example.com', 'doe@example.com'],
            None, None,
            {'subscriber': 'email@example.com'})
        self.assertTrue(form.is_valid())

    def test_required_fields(self):
        # There is no required fields, so empty form should be valid.
        form = ChangeSubscriptionForm(
            ['email@example.com', 'john@example.com', 'doe@example.com'],
            None, None,
            {})
        self.assertTrue(form.is_valid())

    def test_change_subscription_with_user_uuid(self):
        user_emails = ['aperson@example.com']
        form = ChangeSubscriptionForm(
            user_emails,
            '00000000000000000000000000000004',
            'aperson@example.com',
            {'subscriber': '00000000000000000000000000000004'}
        )
        self.assertEqual(form.is_valid(), True)

    def test_subscribe_with_invalid_uuid(self):
        # Test that invalid UUID returns errors.
        user_emails = ['aperson@example.com']
        # UUIDs are 32bit and 4 is not a valid email.
        form = ChangeSubscriptionForm(user_emails,
                                      '4',
                                      'aperson@example.com',
                                      {'subscriber': '4'})
        self.assertEqual(form.is_valid(), False)
        self.assertEqual(form.errors['subscriber'][0],
                         'Invalid: "4" should be either email or UUID')


class TestListNew(TestCase):

    def test_form_fields_list(self):
        domain_choices = [
            ("mailman.most-desirable.org", "mailman.most-desirable.org")]
        style_choices = [
            ("legacy-default", 'Ordinary discussion mailing list style.'),
            ("legacy-announce", 'Announce only mailing list style.')]
        form = ListNew(domain_choices, style_choices,
                       {'listname': 'xyz',
                        'mail_host': 'mailman.most-desirable.org',
                        'list_owner': 'contact@mailman.most-desirable.org',
                        'advertised': 'True',
                        'list_style': 'legacy-default',
                        'description': 'The Most Desirable organization'})
        self.assertTrue(form.is_valid(), form.errors)

    def test_form_fields_list_invalid(self):
        domain_choices = [
            ("mailman.most-desirable.org", "mailman.most-desirable.org")]
        style_choices = [
            ("legacy-default", 'Ordinary discussion mailing list style.'),
            ("legacy-announce", 'Announce only mailing list style.')]
        form = ListNew(domain_choices, style_choices,
                       {'listname': 'xy#z',
                        'mail_host': 'mailman.most-desirable.org',
                        'list_owner': 'mailman.most-desirable.org',
                        'advertised': 'abcd',
                        'list_style': 'defg',
                        'description': 'The Most Desirable organization'})
        self.assertFalse(form.is_valid())
        # Test that all invalid fields are actually checked.
        for field in ('list_owner', 'advertised'):
            self.assertTrue(field in form.errors)
        self.assertTrue('Enter a valid email address.' in
                        form.errors['list_owner'])
        self.assertTrue(
            'Select a valid choice. abcd is not one of the available choices.'
            in form.errors['advertised'])

    def test_form_without_domain_choices(self):
        form = ListNew([],
                       {'listname': 'xyz',
                        'mail_host': 'mailman.most-desirable.org',
                        'list_owner': 'contact@mailman.most-desirable.org',
                        'advertised': 'True',
                        'description': 'The Most Desirable organization', })
        # Without domain choices, the form is not going to be valid.
        self.assertFalse(form.is_valid())
        self.assertTrue(form.fields['mail_host'].help_text ==     # noqa: W504
                        'Site admin has not created any domains')

    def test_listname_validation(self):
        domain_choices = [
            ("mailman.most-desirable.org", "mailman.most-desirable.org")]
        style_choices = [
            ("legacy-default", 'Ordinary discussion mailing list style.'),
            ("legacy-announce", 'Announce only mailing list style.')]
        form = ListNew(domain_choices, style_choices,
                       {'listname': 'xy@z',
                        'mail_host': 'mailman.most-desirable.org',
                        'list_owner': 'mailman.most-desirable.org',
                        'advertised': 'abcd',
                        'description': 'The Most Desirable organization', })
        self.assertFalse(form.is_valid())
        self.assertTrue('listname' in form.errors)
        self.assertTrue('Please enter a valid listname' in
                        form.errors['listname'])

    @unittest.expectedFailure
    def test_listname_validation_errors_sane(self):
        # This test is going to fail right now, but needs to be fixed.
        form = ListNew([
            ("mailman.most-desirable.org", "mailman.most-desirable.org")],
            {'listname': 'xy#z',
             'mail_host': 'mailman.most-desirable.org',
             'list_owner': 'mailman.most-desirable.org',
             'advertised': 'abcd',
             'description': 'The Most Desirable organization', })
        self.assertFalse(form.is_valid())
        self.assertTrue('listname' in form.errors)
        self.assertEqual(
            'Please enter a valid listname, "@" is not allowed in listname',
            form.errors['listname'])

    def test_form_fields_order(self):
        domain_choices = [
            ("mailman.most-desirable.org", "mailman.most-desirable.org")]
        style_choices = [
            ("legacy-default", 'Ordinary discussion mailing list style.'),
            ("legacy-announce", 'Announce only mailing list style.')]
        form = ListNew(domain_choices, style_choices,
                       {'listname': 'xyz',
                        'mail_host': 'mailman.most-desirable.org',
                        'list_owner': 'mailman@most-desirable.org',
                        'list_style': 'legacy-default',
                        'advertised': 'True',
                        'description': 'The Most Desirable organization', })
        self.assertTrue(form.is_valid())
        # The order of the fields should remain exactly like this.
        self.assertEqual(list(form.fields),
                         ['listname',
                          'mail_host',
                          'list_owner',
                          'advertised',
                          'list_style',
                          'description'])


class TestListIdentityForm(TestCase):

    def test_not_required_fields(self):
        # Only advertised is the required form field.
        form = ListIdentityForm({
            'advertised': 'True',
        }, mlist=None)
        self.assertTrue(form.is_valid(), form.errors)

    def test_field_validations(self):
        form = ListIdentityForm({
            'advertised': 'abcd',
            'description': 'This is the most desirable organization',
            'info': 'This is a larger description of this mailing list.',
            'display_name': 'Most Desirable Mailing List',
            'subject_prefix': '  [Most Desirable]               ',
            'preferred_language': 'en',
            'member_roster_visibility': 'public',
        }, mlist=None)
        self.assertFalse(form.is_valid())
        self.assertTrue('advertised' in form.errors)
        self.assertEqual(['Select a valid choice. abcd is not one of the available choices.'],  # noqa
                         form.errors['advertised'])
        # We shouldn't be removing trailing whitespaces, but we
        # should remove the leading ones.
        self.assertEqual(form.cleaned_data['subject_prefix'],
                         '[Most Desirable]               ')


class TestListMassSubscription(TestCase):

    def test_all_valid_email_formats(self):
        form = ListMassSubscription({
            'emails': '''
jdoe@example.com
<jdoe@example.com>
John Doe <jdoe@example.com>
"John Doe" <jdoe@example.com>
jdoe@example.com (John Doe)'''})
        self.assertTrue(form.is_valid())

    def test_required_fields(self):
        form = ListMassSubscription({'email': '   '})
        self.assertFalse(form.is_valid())
        self.assertEqual(['This field is required.'],
                         form.errors['emails'])
        form = ListMassSubscription({'email': '----'})
        self.assertFalse(form.is_valid())
        self.assertEqual(['This field is required.'],
                         form.errors['emails'])


class TestListMassRemoval(TestCase):

    def test_all_valid_formats(self):
        form = ListMassRemoval({
            'emails': '''
jdoe@example.com
<jdoe@example.com>
John Doe <jdoe@example.com>
"John Doe" <jdoe@example.com>
jdoe@example.com (John Doe)'''})
        self.assertTrue(form.is_valid())


class TestAddBanForm(TestCase):

    def test_form_validity(self):
        form = AddBanForm({'email': 'jdoe@example.com'})
        self.assertTrue(form.is_valid())

    def test_missing_fields_errors(self):
        form = AddBanForm({})
        self.assertFalse(form.is_valid())
        self.assertTrue('email' in form.errors)
        self.assertEqual(form.errors['email'],
                         ['Please enter an email address.'])

    @unittest.expectedFailure
    def test_invalid_fields_type(self):
        # Valid values for email is either a regexp or an email address.
        # However, this is currently not validated by the form.
        form = AddBanForm({'email': 'invalid@'})
        self.assertFalse(form.is_valid())
        self.assertTrue('email' in form.errors)
        self.assertEqual(form.errors['email'],
                         ['Please enter a valid email address.'])


class TestListHeaderMatchForm(TestCase):

    def test_form_validity(self):
        # Test by putting only required fields.
        form = ListHeaderMatchForm({
            'header': 'To',
            'pattern': 'value@example.com'})
        self.assertTrue(form.is_valid())
        # Test all fileds.
        form = ListHeaderMatchForm({
            'header': 'To',
            'pattern': 'value@example.com',
            'action': 'accept'})
        self.assertTrue(form.is_valid())
        # Defer is not a valid action, so validation should fail here.
        form = ListHeaderMatchForm({
            'header': 'To',
            'pattern': 'value@example.com',
            'action': 'defer'})
        self.assertFalse(form.is_valid())
        self.assertTrue('action' in form.errors)


class TestMemberModeration(TestCase):

    def test_moderation_action_validity(self):
        form = MemberModeration({'moderation_action': 'moderation'})
        self.assertFalse(form.is_valid())
        self.assertTrue('moderation_action' in form.errors)
        self.assertEqual(form.errors['moderation_action'],
                         ['Select a valid choice. moderation is not one of the available choices.'])    # noqa

    def test_required_fields(self):
        form = MemberModeration({})
        self.assertTrue(form.is_valid())


class TestListAutomaticResponsesForm(TestCase):

    fields = ('autorespond_owner', 'autoresponse_owner_text',
              'autorespond_postings', 'autoresponse_postings_text',
              'autorespond_requests', 'autoresponse_request_text',
              'autoresponse_grace_period', 'send_welcome_message',
              'welcome_message_uri', 'send_goodbye_message',
              'goodbye_message_uri', 'admin_immed_notify',
              'admin_notify_mchanges')

    def prepare_formdata(self, values):
        return dict(((key, val) for key, val in zip(self.fields, values) if val is not None))   # noqa

    def test_required_fields_only(self):
        values = ('respond_and_continue', None,
                  'respond_and_continue', None,
                  'respond_and_continue', None,
                  '2', None,
                  None, None,
                  None, None)
        formdata = self.prepare_formdata(values)
        form = ListAutomaticResponsesForm(formdata, mlist=None)
        self.assertTrue(form.is_valid())

    def test_all_values(self):
        values = ('respond_and_continue', 'Autorespond text',
                  'respond_and_continue', 'Autorespond text',
                  'respond_and_continue', 'Autorespond text',
                  '2', 'True',
                  'http://example.com/welcome_text',
                  'True',
                  'http://example.com/goodbye_message',
                  'True', 'False')
        formdata = self.prepare_formdata(values)
        form = ListAutomaticResponsesForm(formdata, mlist=None)
        print(form.errors)
        self.assertTrue(form.is_valid())


class TestAlterMessageForm(TestCase):

    fields = ('filter_content', 'collapse_alternatives',
              'convert_html_to_plaintext', 'anonymous_list',
              'include_rfc2369_headers', 'allow_list_posts',
              'reply_list_posts', 'reply_to_address',
              'first_strip_reply_to', 'reply_goes_to_list',
              'posting_pipeline')

    def prepare_formdata(self, values):
        return dict(((key, val) for key, val in zip(self.fields, values) if val is not None))   # noqa


class TestDMARCMitigationsForm(TestCase):

    def test_required_fields(self):
        # All fields in the form are optional, so an empty form should be
        # valid.
        form = DMARCMitigationsForm({}, mlist=None)
        self.assertTrue(form.is_valid())

    def test_all_fields(self):
        formdata = dict(
            dmarc_mitigate_action='munge_from',
            dmarc_mitigate_unconditionally='True',
            dmarc_moderation_notice='This is a moderation notice',
            dmarc_wrapped_message_text='This is wrapped message text')
        form = DMARCMitigationsForm(formdata, mlist=None)
        self.assertTrue(form.is_valid())


class TestDigestSettingsForm(TestCase):

    def test_required_fields(self):
        form = DigestSettingsForm({}, mlist=None)
        self.assertFalse(form.is_valid())
        self.assertTrue('digest_size_threshold' in form.errors)
        self.assertEqual(form.errors['digest_size_threshold'],
                         ['This field is required.'])
        form = DigestSettingsForm({'digest_size_threshold': 40}, mlist=None)
        self.assertTrue(form.is_valid())

    def test_all_fields(self):
        formdata = dict(
            digests_enabled='True',
            digests_send_periodic='True',
            digests_volume_frequency='daily',
            digest_size_threshold='10',
        )
        form = DigestSettingsForm(formdata, mlist=None)
        self.assertTrue(form.is_valid())


class TestMessageAcceptanceForm(TestCase):

    fields = ('acceptable_aliases', 'require_explicit_destination',
              'administrivia', 'default_member_action',
              'default_nonmember_action', 'max_message_size',
              'max_num_recipients')

    def prepare_formdata(self, values):
        return dict(((key, val) for key, val in zip(self.fields, values) if val is not None))   # noqa

    def test_required_fields(self):
        # Without any fields, form should not be valid.
        form = MessageAcceptanceForm({}, mlist=None)
        self.assertFalse(form.is_valid())
        # Now lets try with only required fields.
        values = (None, None, None, 'hold', 'hold', 40, 100)
        form = MessageAcceptanceForm(self.prepare_formdata(values), mlist=None)
        print(form.errors)
        self.assertTrue(form.is_valid())

    def test_all_fields(self):
        pass


class TestArchiveSettingsForm(TestCase):

    def setUp(self):
        self.mlist = create_mock_list()
        archivers = {'pipermail': True, 'hyperkitty': True}
        self.mlist.archivers.keys.side_effect = archivers.keys
        self.mlist.archivers.__getitem__.side_effect = archivers.__getitem__
        self.mlist.archivers.__iter__.side_effect = archivers.__iter__
        self.mlist.archivers.__contains__.side_effect = archivers.__contains__

    def test_required_fields(self):
        # First try without any fields.
        form = ArchiveSettingsForm({}, mlist=self.mlist)
        self.assertFalse(form.is_valid())
        self.assertTrue('archive_policy' in form.errors)
        # Now, with only required fields, this should be a valid form.
        form = ArchiveSettingsForm(dict(archive_policy='public'),
                                   mlist=self.mlist)
        self.assertTrue(form.is_valid())

    def test_all_fields(self):
        formdata = dict(archive_policy='public',
                        archivers=['pipermail', 'hyperkitty'])
        form = ArchiveSettingsForm(formdata, mlist=self.mlist)
        self.assertTrue(form.is_valid())

    def test_setup_archivers_populated(self):
        formdata = dict(archive_policy='public',
                        archivers=['pipermail', 'hyperkitty'])
        form = ArchiveSettingsForm(formdata, mlist=self.mlist,
                                   initial={'archivers': None})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.initial['archivers'],
                         ['hyperkitty', 'pipermail'])


class TestMemberPolicyForm(TestCase):

    def test_required_fields(self):
        form = MemberPolicyForm({}, mlist=None)
        self.assertFalse(form.is_valid())
        self.assertTrue('subscription_policy' in form.errors)
        self.assertTrue('unsubscription_policy' in form.errors)
        form = MemberPolicyForm(
            dict(subscription_policy='confirm',
                 unsubscription_policy='confirm'),
            mlist=None)
        self.assertTrue(form.is_valid())


class TestListAnonymousSubscribe(TestCase):

    def test_required_fields_only(self):
        form = ListAnonymousSubscribe(dict(email='bob@exmaple.com'))
        self.assertTrue(form.is_valid())

    def test_email_is_validated(self):
        form = ListAnonymousSubscribe(dict(email='invalid'))
        self.assertFalse(form.is_valid())
        self.assertTrue('email' in form.errors)
        self.assertEqual(form.errors['email'],
                         ['Please enter a valid email address.'])

    def test_all_fields(self):
        form = ListAnonymousSubscribe(dict(email='bob@example.com',
                                           display_name='Bob'))
        self.assertTrue(form.is_valid())



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from unittest.mock import patch

from django.contrib.auth.models import AnonymousUser, User
from django.core.exceptions import PermissionDenied
from django.test import TestCase
from django.test.client import RequestFactory

from allauth.account.models import EmailAddress
from mailmanclient import Client

from postorius.auth.decorators import (
    list_moderator_required, list_owner_required, superuser_required)
from postorius.tests.utils import create_mock_list


@list_owner_required
def dummy_function(request, list_id):
    return True


@list_moderator_required
def dummy_function_mod_req(request, list_id):
    return True


@superuser_required
def dummy_superuser_required(request):
    return True


def create_user():
    user = User.objects.create_user(
        'les c', 'les@primus.org', 'pwd')
    EmailAddress.objects.create(
        user=user, email=user.email, verified=True)
    return user


class MockMember():

    def __init__(self, email):
        self.email = email


class ListOwnerRequiredTest(TestCase):
    """Tests the list_owner_required auth decorator."""

    def setUp(self):
        from postorius.tests.utils import create_mock_list
        self.request_factory = RequestFactory()
        # create a mock list with members
        list_name = 'foolist.example.org'
        list_id = 'foolist.example.org'
        self.mock_list = create_mock_list(dict(
            fqdn_listname=list_name,
            list_id=list_id))

    @patch.object(Client, 'get_list')
    def test_not_authenticated(self, mock_get_list):
        """Should raise PermissionDenied if user is not authenticated."""
        mock_get_list.return_value = self.mock_list
        request = self.request_factory.get(
            '/lists/foolist.example.org/settings/')
        request.user = AnonymousUser()
        self.assertRaises(PermissionDenied, dummy_function, request,
                          list_id='foolist.example.org')

    @patch.object(Client, 'get_list')
    def test_superuser(self, mock_get_list):
        """Should call the dummy method, if user is superuser."""
        mock_get_list.return_value = self.mock_list
        request = self.request_factory.get(
            '/lists/foolist.example.org/settings/')
        request.user = User.objects.create_superuser(
            'su1', 'su@sodo.org', 'pwd')
        EmailAddress.objects.create(
            user=request.user, email=request.user.email, verified=True)
        return_value = dummy_function(
            request, list_id='foolist.example.org')
        self.assertEqual(return_value, True)

    @patch.object(Client, 'get_list')
    def test_non_list_owner(self, mock_get_list):
        """Should raise PermissionDenied if user is not a list owner."""
        # prepare mock list object
        self.mock_list.owners = [MockMember('geddy@rush.it')]
        mock_get_list.return_value = self.mock_list
        # prepare request
        request = self.request_factory.get(
            '/lists/foolist.example.org/settings/')
        request.user = create_user()
        self.assertRaises(PermissionDenied, dummy_function, request,
                          list_id='foolist.example.org')

    @patch.object(Client, 'get_list')
    def test_list_owner(self, mock_get_list):
        """Should return fn return value if user is the list owner."""
        # prepare mock list object
        self.mock_list.owners = [MockMember('les@primus.org')]
        mock_get_list.return_value = self.mock_list
        # prepare request
        request = self.request_factory.get(
            '/lists/foolist.example.org/settings/')
        request.user = create_user()
        return_value = dummy_function(
            request, list_id='foolist.example.org')
        self.assertEqual(return_value, True)


class ListModeratorRequiredTest(TestCase):
    """Tests the list_owner_required auth decorator."""

    def setUp(self):
        from postorius.tests.utils import create_mock_list
        self.request_factory = RequestFactory()
        # create a mock list with members
        list_name = 'foolist.example.org'
        list_id = 'foolist.example.org'
        self.mock_list = create_mock_list(dict(
            fqdn_listname=list_name,
            list_id=list_id))

    @patch.object(Client, 'get_list')
    def test_not_authenticated(self, mock_get_list):
        """Should raise PermissionDenied if user is not authenticated."""
        mock_get_list.return_value = self.mock_list
        request = self.request_factory.get(
            '/lists/foolist.example.org/settings/')
        request.user = AnonymousUser()
        self.assertRaises(PermissionDenied, dummy_function_mod_req, request,
                          list_id='foolist.example.org')

    @patch.object(Client, 'get_list')
    def test_superuser(self, mock_get_list):
        """Should call the dummy method, if user is superuser."""
        mock_get_list.return_value = self.mock_list
        request = self.request_factory.get(
            '/lists/foolist.example.org/settings/')
        request.user = User.objects.create_superuser(
            'su2', 'su@sodo.org', 'pwd')
        EmailAddress.objects.create(
            user=request.user, email=request.user.email, verified=True)
        return_value = dummy_function_mod_req(
            request, list_id='foolist.example.org')
        self.assertEqual(return_value, True)

    @patch.object(Client, 'get_list')
    def test_non_list_moderator(self, mock_get_list):
        """Should raise PermissionDenied if user is not a list owner."""
        # prepare mock list object
        self.mock_list.moderators = [MockMember('geddy@rush.it')]
        mock_get_list.return_value = self.mock_list
        # prepare request
        request = self.request_factory.get(
            '/lists/foolist.example.org/settings/')
        request.user = create_user()
        self.assertRaises(PermissionDenied, dummy_function_mod_req, request,
                          list_id='foolist.example.org')

    @patch.object(Client, 'get_list')
    def test_list_owner(self, mock_get_list):
        """Should return fn return value if user is the list owner."""
        # prepare mock list object
        self.mock_list.owners = [MockMember('les@primus.org')]
        mock_get_list.return_value = self.mock_list
        # prepare request
        request = self.request_factory.get(
            '/lists/foolist.example.org/settings/')
        request.user = create_user()
        return_value = dummy_function_mod_req(
            request, list_id='foolist.example.org')
        self.assertEqual(return_value, True)

    @patch.object(Client, 'get_list')
    def test_list_moderator(self, mock_get_list):
        """Should return fn return value if user is the list moderator."""
        # prepare mock list object
        self.mock_list.moderators = [MockMember('les@primus.org')]
        mock_get_list.return_value = self.mock_list
        # prepare request
        request = self.request_factory.get(
            '/lists/foolist.example.org/settings/')
        request.user = create_user()
        return_value = dummy_function_mod_req(
            request, list_id='foolist.example.org')
        self.assertEqual(return_value, True)


class TestSuperUserOr403(TestCase):
    """Tests superuser_required auth decorator"""

    def setUp(self):
        self.request_factory = RequestFactory()
        # create a mock list with members
        list_name = 'foolist.example.org'
        list_id = 'foolist.example.org'
        self.mock_list = create_mock_list(dict(
            fqdn_listname=list_name,
            list_id=list_id))

    @patch.object(Client, 'get_list')
    def test_anonymous_user(self, mock_get_list):
        """Should raise PermissionDenied if user is an anonymous user."""
        mock_get_list.return_value = self.mock_list

        request = self.request_factory.get(
            '/lists/foolist.example.org/settings/')
        request.user = AnonymousUser()
        self.assertRaises(PermissionDenied, dummy_superuser_required, request)

    @patch.object(Client, 'get_list')
    def test_normal_user(self, mock_get_list):
        """Should raise PermissionDenied if user is a normal user."""
        mock_get_list.return_value = self.mock_list

        request = self.request_factory.get(
            '/lists/foolist.example.org/settings/')
        request.user = create_user()
        self.assertRaises(PermissionDenied, dummy_superuser_required, request)

    @patch.object(Client, 'get_list')
    def test_super_user(self, mock_get_list):
        """Should not raise PermissionDenied if user is superuser."""
        mock_get_list.return_value = self.mock_list

        request = self.request_factory.get(
            '/lists/foolist.example.org/settings/')
        request.user = create_user()
        request.user.is_superuser = True
        request.user.save()
        self.assertTrue(dummy_superuser_required(request))



# -*- coding: utf-8 -*-
# Copyright (C) 2018-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#

from django.contrib.auth.models import AnonymousUser
from django.http import HttpResponse
from django.test import RequestFactory, TestCase

from postorius.views.generic import MailingListView


class TestGenericView(TestCase):
    """Test generic view adds templates and Mailing List."""

    def setUp(self):
        # We first create a test view that is based on the MailingListView.
        # This view only implements a POST and no GET.
        class TestView(MailingListView):
            def post(self, request):
                return HttpResponse(status=200)

        self.view = TestView.as_view()
        self.factory = RequestFactory()

    def test_no_GET_in_view(self):
        request = self.factory.post('/')
        request.user = AnonymousUser()
        # Since out view implements a POST, we shouldn't get an ERROR here.
        response = self.view(request)
        self.assertEqual(response.status_code, 200)
        # Now, let's try a GET request.
        request = self.factory.get('/')
        request.user = AnonymousUser()
        # We shouldn't a ImproperlyConfigured error with GET but 405 error.
        response = self.view(request)
        self.assertEqual(response.status_code, 405)



# Copyright (C) 2019-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius
#
# mailman.client is free software: you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation, version 3 of the License.
#
# mailman.client is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
# License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with mailman.client.  If not, see <http://www.gnu.org/licenses/>.

"""Testing utilities."""

import os
import shutil
import tempfile
from textwrap import dedent

import pytest
from mailman.config import config
from mailman.core import initialize
from mailman.core.initialize import INHIBIT_CONFIG_FILE
from mailman.testing.helpers import (
    TestableMaster, reset_the_world, wait_for_webservice)


@pytest.fixture(scope='session')
def config_initialize(request):
    # Set up the basic configuration stuff.  Turn off path creation until
    # we've pushed the testing config.
    config.create_paths = False
    initialize.initialize_1(INHIBIT_CONFIG_FILE)
    # Calculate a temporary VAR_DIR directory so that run-time artifacts
    # of the tests won't tread on the installation's data.  This also
    # makes it easier to clean up after the tests are done, and insures
    # isolation of test suite runs.
    var_dir = tempfile.mkdtemp()
    # We need a test configuration both for the foreground process and any
    # child processes that get spawned.  lazr.config would allow us to do
    # it all in a string that gets pushed, and we'll do that for the
    # foreground, but because we may be spawning processes (such as
    # runners) we'll need a file that we can specify to the with the -C
    # option.  Craft the full test configuration string here, push it, and
    # also write it out to a temp file for -C.
    #
    # Create a dummy postfix.cfg file so that the test suite doesn't try
    # to run the actual postmap command, which may not exist anyway.
    postfix_cfg = os.path.join(var_dir, 'postfix.cfg')
    with open(postfix_cfg, 'w') as fp:
        print(dedent("""
        [postfix]
        postmap_command: true
        transport_file_type: hash
        """), file=fp)
        test_config = dedent("""
        [mailman]
        layout: testing
        [paths.testing]
        var_dir: {}
        [mta]
        configuration: {}
        [devmode]
        enabled: yes
        testing: yes
        recipient: you@yourdomain.com

        [mta]
        smtp_port: 9025
        lmtp_port: 9024
        incoming: mailman.testing.mta.FakeMTA

        [webservice]
        port: 9001

        [archiver.mhonarc]
        enable: yes

        [archiver.mail_archive]
        enable: yes

        [archiver.prototype]
        enable: yes
        """.format(var_dir, postfix_cfg))
        config.create_paths = True
        config.push('test config', test_config)
        # Initialize everything else.
    initialize.initialize_2(testing=True)
    initialize.initialize_3()

    config_file = os.path.join(var_dir, 'test.cfg')
    with open(config_file, 'w') as fp:
        fp.write(test_config)
        print(file=fp)
        config.filename = config_file
    # Start the Mailman's test runner.
    server = TestableMaster(wait_for_webservice)
    server.start('rest', 'in')
    request.addfinalizer(server.stop)
    # Run the test.
    yield
    reset_the_world()
    # Destroy the test database after the tests are done so that there is
    # no data in case the tests are rerun with a database layer like mysql
    # or postgresql which are not deleted in teardown.
    shutil.rmtree(var_dir)
    # Prevent the bit of post-processing on the .pop() that creates
    # directories.  We're basically shutting down everything and we don't
    # need the directories created.  Plus, doing so leaves a var directory
    # turd in the source tree's top-level directory.  We do it this way
    # rather than shutil.rmtree'ing the resulting var directory because
    # it's possible the user created a valid such directory for
    # operational or test purposes.
    config.create_paths = False
    config.pop('test config')


@pytest.fixture(scope='class', autouse=True)
def mailman_rest_layer(config_initialize, request):
    yield
    request.addfinalizer(reset_the_world)



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from unittest.mock import patch

from django.contrib.auth.models import User
from django.urls import reverse

from allauth.account.models import EmailAddress

from postorius.tests.utils import ViewTestCase


class TestSubscription(ViewTestCase):
    """Tests subscription to lists"""

    def setUp(self):
        super(TestSubscription, self).setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.open_list = self.domain.create_list('open_list')
        # Set subscription policy to open
        settings = self.open_list.settings
        settings['subscription_policy'] = 'open'
        settings.save()
        self.mod_list = self.domain.create_list('moderate_subs')
        # Set subscription policy to moderate
        settings = self.mod_list.settings
        settings['subscription_policy'] = 'moderate'
        settings.save()
        # Create django user.
        self.user = User.objects.create_user(
            'testuser', 'test@example.com', 'pwd')
        EmailAddress.objects.create(
            user=self.user, email=self.user.email, verified=True)
        EmailAddress.objects.create(
            user=self.user, email='fritz@example.org', verified=True)
        # Create Mailman user
        self.mm_user = self.mm_client.create_user('test@example.com', '')
        self.mm_user.add_address('fritz@example.org')
        for address in self.mm_user.addresses:
            address.verify()

    @patch('mailmanclient.MailingList.subscribe')
    def test_anonymous_subscribe(self, mock_subscribe):
        response = self.client.post(
            reverse('list_anonymous_subscribe',
                    args=('open_list.example.com', )),
            {'email': 'test@example.com', 'display_name': 'Test User'})
        mock_subscribe.assert_called_once()
        mock_subscribe.assert_called_with(
            'test@example.com', 'Test User', pre_verified=False,
            pre_confirmed=False)
        self.assertRedirects(
            response, reverse('list_summary',
                              args=('open_list.example.com', )))
        self.assertHasSuccessMessage(response)

    def test_subscribe_open(self):
        # The subscription goes straight through.
        self.client.login(username='testuser', password='pwd')
        response = self.client.post(
            reverse('list_subscribe', args=('open_list.example.com', )),
            {'subscriber': 'test@example.com'})
        self.assertEqual(len(self.open_list.members), 1)
        self.assertEqual(len(self.open_list.requests), 0)
        self.assertRedirects(
            response, reverse('list_summary',
                              args=('open_list.example.com', )))
        self.assertHasSuccessMessage(response)

    def test_secondary_open(self):
        # Subscribe with a secondary email address
        self.client.login(username='testuser', password='pwd')
        response = self.client.post(
            reverse('list_subscribe', args=('open_list.example.com', )),
            {'subscriber': 'fritz@example.org'})
        self.assertEqual(len(self.open_list.members), 1)
        self.assertEqual(len(self.open_list.requests), 0)
        self.assertRedirects(
            response, reverse('list_summary',
                              args=('open_list.example.com', )))
        self.assertHasSuccessMessage(response)

    def test_unknown_address(self):
        # Impossible to register with an unknown address
        self.client.login(username='testuser', password='pwd')
        response = self.client.post(
            reverse('list_subscribe', args=('open_list.example.com', )),
            {'subscriber': 'unknown@example.org'})
        self.assertEqual(len(self.open_list.members), 0)
        self.assertEqual(len(self.open_list.requests), 0)
        self.assertRedirects(
            response, reverse('list_summary',
                              args=('open_list.example.com', )))
        self.assertHasErrorMessage(response)

    def test_banned_address(self):
        # Impossible to register with a banned address
        self.client.login(username='testuser', password='pwd')
        self.open_list.bans.add('test@example.com')
        response = self.client.post(
            reverse('list_subscribe', args=('open_list.example.com', )),
            {'subscriber': 'test@example.com'})
        self.assertEqual(len(self.open_list.members), 0)
        self.assertEqual(len(self.open_list.requests), 0)
        self.assertRedirects(
            response, reverse('list_summary',
                              args=('open_list.example.com', )))
        self.assertHasErrorMessage(response)

    def test_subscribe_mod(self):
        # The subscription is held for approval.
        self.client.login(username='testuser', password='pwd')
        response = self.client.post(
            reverse('list_subscribe', args=('moderate_subs.example.com', )),
            {'subscriber': 'test@example.com'})
        self.assertEqual(len(self.mod_list.members), 0)
        self.assertEqual(len(self.mod_list.requests), 1)
        self.assertRedirects(
            response, reverse('list_summary',
                              args=('moderate_subs.example.com', )))
        self.assertHasSuccessMessage(response)

    def test_secondary_mod(self):
        # Subscribe with a secondary email address
        self.client.login(username='testuser', password='pwd')
        response = self.client.post(
            reverse('list_subscribe', args=('moderate_subs.example.com', )),
            {'subscriber': 'fritz@example.org'})
        self.assertEqual(len(self.mod_list.members), 0)
        self.assertEqual(len(self.mod_list.requests), 1)
        self.assertRedirects(
            response, reverse('list_summary',
                              args=('moderate_subs.example.com', )))
        self.assertHasSuccessMessage(response)

    def test_subscribe_already_pending(self):
        # The user tries to subscribe twice on a moderated list.
        self.client.login(username='testuser', password='pwd')
        response = self.client.post(
            reverse('list_subscribe', args=('moderate_subs.example.com', )),
            {'subscriber': 'test@example.com'})
        self.assertEqual(len(self.mod_list.members), 0)
        self.assertEqual(len(self.mod_list.requests), 1)
        self.assertHasSuccessMessage(response)
        # Try to subscribe a second time.
        response = self.client.post(
            reverse('list_subscribe', args=('moderate_subs.example.com', )),
            {'subscriber': 'test@example.com'})
        self.assertEqual(len(self.mod_list.members), 0)
        self.assertEqual(len(self.mod_list.requests), 1)
        message = self.assertHasErrorMessage(response)
        self.assertIn('Subscription request already pending', message)

    def test_subscribe_with_name(self):
        owner = User.objects.create_user(
            'testowner', 'owner@example.com', 'pwd')
        EmailAddress.objects.create(
            user=owner, email=owner.email, verified=True)
        self.open_list.add_owner('owner@example.com')
        self.client.login(username='testowner', password='pwd')
        email_list = """First Person <test-1@example.org>\n
                        "Second Person" <test-2@example.org>\n
                        test-3@example.org (Third Person)\n
                        test-4@example.org\n
                        <test-5@example.org>\n"""
        self.client.post(
            reverse('mass_subscribe', args=('open_list.example.com',)),
            {'emails': email_list,
             'pre_verified': True,
             'send_welcome_message': 'default'})
        self.assertEqual(len(self.open_list.members), 5)
        first = self.open_list.get_member('test-1@example.org')
        second = self.open_list.get_member('test-2@example.org')
        third = self.open_list.get_member('test-3@example.org')
        fourth = self.open_list.get_member('test-4@example.org')
        fifth = self.open_list.get_member('test-5@example.org')
        self.assertEqual(first.address.display_name, 'First Person')
        self.assertEqual(second.address.display_name, 'Second Person')
        self.assertEqual(third.address.display_name, 'Third Person')
        self.assertIsNone(fourth.address.display_name)
        self.assertIsNone(fifth.address.display_name)

    def test_mass_subscribe_send_welcome_message(self):
        owner = User.objects.create_user(
            'testowner', 'owner@example.com', 'pwd')
        EmailAddress.objects.create(
            user=owner, email=owner.email, verified=True)
        self.open_list.add_owner('owner@example.com')
        self.client.login(username='testowner', password='pwd')
        virgin_q = self.mm_client.queues['virgin']
        initial_files = len(virgin_q.files)
        email_list = """First Person <test-1@example.org>\n
                        "Second Person" <test-2@example.org>\n"""
        self.client.post(
            reverse('mass_subscribe', args=('open_list.example.com',)),
            {'emails': email_list,
             'pre_verified': True, 'send_welcome_message': True})
        self.assertEqual(len(self.open_list.members), 2)
        virgin_q = self.mm_client.queues['virgin']
        # There should be two more files in the queue.
        self.assertEqual(len(virgin_q.files) - initial_files, 2)
        initial_files += 2
        # Now subscribe some users by changing it to No.
        email_list = """test-3@example.org (Third Person)\n"""
        self.client.post(
            reverse('mass_subscribe', args=('open_list.example.com',)),
            {'emails': email_list,
             'pre_verified': True, 'send_welcome_message': False})
        self.assertEqual(len(self.open_list.members), 3)
        # There should be zero more messages in virgin queue because we set
        # `sent_welcome_message` to False.
        # There should be two more files in the queue.
        virgin_q = self.mm_client.queues['virgin']
        self.assertEqual(len(virgin_q.files) - initial_files, 0)
        # If set to none, the default value of send_welcome_message works,
        # which is True.
        email_list = """test4@example.org (Third Person)\n"""
        self.client.post(
            reverse('mass_subscribe', args=('open_list.example.com',)),
            {'emails': email_list,
             'pre_verified': True, 'send_welcome_message': 'default'})
        self.assertEqual(len(self.open_list.members), 4)
        virgin_q = self.mm_client.queues['virgin']
        self.assertEqual(len(virgin_q.files) - initial_files, 1)

    def test_change_subscription_open(self):
        # The subscription is changed from an address to another
        self.open_list.subscribe('test@example.com')
        self.assertEqual(len(self.open_list.members), 1)
        self.assertEqual(len(self.open_list.requests), 0)
        self.client.login(username='testuser', password='pwd')
        response = self.client.post(
            reverse('change_subscription', args=['open_list.example.com']),
            {'subscriber': 'fritz@example.org'})
        self.assertHasSuccessMessage(response)
        self.assertEqual(len(self.open_list.members), 1)
        self.assertEqual(len(self.open_list.requests), 0)
        try:
            member = self.open_list.get_member('fritz@example.org')
        except ValueError:
            self.fail('The subscription was not changed')
        self.assertEqual(member.email, 'fritz@example.org')
        self.assertRedirects(
            response, reverse('list_summary',
                              args=('open_list.example.com', )))

    def test_change_subscription_confirm(self):
        # The subscription is changed from an address to another
        confirm_list = self.domain.create_list('confirm_list')
        settings = confirm_list.settings
        settings['subscription_policy'] = 'confirm'
        settings.save()
        confirm_list.subscribe('test@example.com', pre_confirmed=True)
        self.assertEqual(len(confirm_list.members), 1)
        self.assertEqual(len(confirm_list.requests), 0)
        self.client.login(username='testuser', password='pwd')
        response = self.client.post(
            reverse('change_subscription', args=['confirm_list.example.com']),
            {'subscriber': 'fritz@example.org'})
        self.assertHasSuccessMessage(response)
        self.assertEqual(len(confirm_list.members), 1)
        self.assertEqual(len(confirm_list.requests), 0)
        try:
            member = confirm_list.get_member('fritz@example.org')
        except ValueError:
            self.fail('The subscription was not changed')
        self.assertEqual(member.email, 'fritz@example.org')
        self.assertRedirects(
            response, reverse('list_summary',
                              args=('confirm_list.example.com', )))



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django.contrib.messages.storage.base import BaseStorage, Message
from django.test import TestCase
from django.test.client import RequestFactory

from mailmanclient import MailingList

from postorius.views.list import ListMembersViews


class LocalStorage(BaseStorage):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._storage = []

    def _get(self, *args, **kwargs):
        return self._storage, True

    def _store(self, messages, response, *args, **kwargs):
        if messages:
            self._storage = messages
        else:
            self._storage = []
        return []


class TestListView(TestCase):
    def setUp(self):
        self.request_factory = RequestFactory()

    def test_error_message(self):
        mailing_list_data = {'list_id': 'test_list.example.com'}
        mailing_list = MailingList(None, None, data=mailing_list_data)
        view = ListMembersViews(mailing_list=mailing_list)
        request_data = {
            'email': ['.*@example.com'],
            'display_name': ['']
        }
        request = self.request_factory.post(
            '/postorius/lists/test_list.example.com/members/nonmember/',
            data=request_data
        )
        request._messages = LocalStorage(request)
        view.post(request, 'test_list.example.com', role='nonmember')
        self.assertTrue(len(request._messages))
        self.assertIsInstance(list(request._messages)[0], Message)



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django.contrib.auth.models import User
from django.urls import reverse

from allauth.account.models import EmailAddress

from postorius.tests.utils import ViewTestCase


class ListCreationTest(ViewTestCase):
    """Tests for the new list page."""

    def setUp(self):
        super(ListCreationTest, self).setUp()
        self.user = User.objects.create_user('user', 'user@example.com', 'pwd')
        self.superuser = User.objects.create_superuser('su', 'su@example.com',
                                                       'pwd')
        for user in (self.user, self.superuser):
            EmailAddress.objects.create(
                user=user, email=user.email, verified=True)
        self.domain = self.mm_client.create_domain('example.com')

    def test_permission_denied(self):
        self.client.login(username='user', password='pwd')
        response = self.client.get(reverse('list_new'))
        self.assertEqual(response.status_code, 403)

    def test_duplicate_listname_validation(self):
        # First, let's create a mailing list.
        self.domain.create_list('foo')
        self.client.login(username='su', password='pwd')
        post_data = {'listname': 'foo',
                     'mail_host': 'example.com',
                     'list_owner': 'owner@example.com',
                     'advertised': 'True',
                     'list_style': 'legacy-default',
                     'description': 'A new list.'}
        response = self.client.post(reverse('list_new'), post_data)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Mailing List already exists')
        # Make sure this is the list create form and not just error page.
        self.assertContains(response, 'Create a new list')

    def test_new_list_created_with_owner(self):
        self.client.login(username='su', password='pwd')
        post_data = {'listname': 'a_new_list',
                     'mail_host': 'example.com',
                     'list_owner': 'owner@example.com',
                     'advertised': 'True',
                     'list_style': 'legacy-default',
                     'description': 'A new list.'}
        response = self.client.post(reverse('list_new'), post_data)
        self.assertEqual(response.status_code, 302)
        # self.assertHasSuccessMessage(response)
        a_new_list = self.mm_client.get_list('a_new_list@example.com')
        self.assertEqual(a_new_list.fqdn_listname, 'a_new_list@example.com')
        owner_emails = [owner.email for owner in a_new_list.owners]
        self.assertEqual(owner_emails, ['owner@example.com'])

    def test_listname_validation(self):
        self.client.login(username='su', password='pwd')
        post_data = {'listname': 'a new list',
                     'mail_host': 'example.com',
                     'list_owner': 'owner@example.com',
                     'advertised': 'True',
                     'list_style': 'legacy-default',
                     'description': 'A new list.'}
        response = self.client.post(reverse('list_new'), post_data)
        self.assertEqual(response.status_code, 200)
        # self.assertHasErrorMessage(response)
        self.assertContains(response, 'Please enter a valid listname')



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#


from django import forms
from django.utils.encoding import smart_text
from django.utils.translation import gettext_lazy as _


class ListOfStringsField(forms.Field):
    widget = forms.widgets.Textarea

    def prepare_value(self, value):
        if isinstance(value, list):
            value = '\n'.join(value)
        return value

    def to_python(self, value):
        "Returns a list of Unicode object."
        if value in self.empty_values:
            return []
        result = []
        for line in value.splitlines():
            line = line.strip()
            if not line:
                continue
            result.append(smart_text(line))
        return result


class NullBooleanRadioSelect(forms.RadioSelect):
    """
    This is necessary to detect that such a field has not been changed.
    """

    def value_from_datadict(self, data, files, name):
        value = data.get(name, None)
        return {'2': True,
                True: True,
                'True': True,
                '3': False,
                'False': False,
                False: False}.get(value, None)


class SiteModelChoiceField(forms.ModelChoiceField):

    def label_from_instance(self, obj):
        return "%s (%s)" % (obj.name, obj.domain)


class MultipleChoiceForm(forms.Form):

    class MultipleChoiceField(forms.MultipleChoiceField):

        def validate(self, value):
            pass

    choices = MultipleChoiceField(
        widget=forms.CheckboxSelectMultiple,
    )

    def clean_choices(self):
        if len(self.cleaned_data['choices']) < 1:
            raise forms.ValidationError(_('Make at least one selection'))
        return self.cleaned_data['choices']



# -*- coding: utf-8 -*-
# Copyright (C) 1998-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


import logging

from django.conf import settings
from django.shortcuts import render
from django.utils.translation import gettext as _

from allauth.account.models import EmailAddress
from mailmanclient import Client


logger = logging.getLogger(__name__)


def render_api_error(request):
    """Renders an error template.
    Use if MailmanApiError is catched.
    """
    return render(request, 'postorius/errors/generic.html',
                  {'error': _('Mailman REST API not available. Please start Mailman core.')},   # noqa: E501
                  status=503)


def render_client_error(request, error):
    return render(request, 'postorius/errors/generic.html',
                  {'error': str(error)},
                  status=error.code)


def get_mailman_client():
    # easier to patch during unit tests
    client = Client(
        '%s/3.1' %
        settings.MAILMAN_REST_API_URL,
        settings.MAILMAN_REST_API_USER,
        settings.MAILMAN_REST_API_PASS)
    return client


def with_empty_choice(choices):
    """Add an empty Choice for unset values in dropdown."""
    return [(None, '-----')] + list(choices)


def set_preferred(user, mm_user):
    """Set preferred address in Mailman Core.

    :param user: The Django user mode to set preferred address.
    :param mm_user: The Mailman User object to set preferred address for.
    """
    client = get_mailman_client()
    primary_email = EmailAddress.objects.get_primary(user)
    if primary_email is not None and primary_email.verified:
        # First, make sure that the email address is verified in Core,
        # otherwise, we can't set it as a primary address.
        addr = client.get_address(primary_email.email)
        if not addr.verified_on:
            addr.verify()
        mm_user.preferred_address = primary_email.email
        return primary_email.email
    return None


def get_member_or_nonmember(mlist, email):
    """Return either a Member or a Non-member with `email` in mlist.

    :param mlist: MailingList object to get membership for.
    :param email: Email address of the member or nonmember.
    :returns: Member if found otherwise None.
    """
    try:
        member = mlist.get_member(email)
    except ValueError:
        # Not a Member, try getting non-member.
        try:
            member = mlist.get_nonmember(email)
        except ValueError:
            member = None
    return member


LANGUAGES = (
    ('ar', 'Arabic'),
    ('ast', 'Asturian'),
    ('ca', 'Catalan'),
    ('cs', 'Czech'),
    ('da', 'Danish'),
    ('de', 'German'),
    ('el', 'Greek'),
    ('es', 'Spanish'),
    ('et', 'Estonian'),
    ('eu', 'Euskara'),
    ('fi', 'Finnish'),
    ('fr', 'French'),
    ('gl', 'Galician'),
    ('he', 'Hebrew'),
    ('hr', 'Croatian'),
    ('hu', 'Hungarian'),
    ('ia', 'Interlingua'),
    ('it', 'Italian'),
    ('ja', 'Japanese'),
    ('ko', 'Korean'),
    ('lt', 'Lithuanian'),
    ('nl', 'Dutch'),
    ('no', 'Norwegian'),
    ('pl', 'Polish'),
    ('pt', 'Protuguese'),
    ('pt_BR', 'Protuguese (Brazil)'),
    ('ro', 'Romanian'),
    ('ru', 'Russian'),
    ('sk', 'Slovak'),
    ('sl', 'Slovenian'),
    ('sr', 'Serbian'),
    ('sv', 'Swedish'),
    ('tr', 'Turkish'),
    ('uk', 'Ukrainian'),
    ('vi', 'Vietnamese'),
    ('zh_CN', 'Chinese'),
    ('zh_TW', 'Chinese (Taiwan)'),
    ('en', 'English (USA)'),
)



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

from unittest.mock import MagicMock, patch
from urllib.error import HTTPError

from django.contrib.auth.models import User
from django.test import override_settings

from allauth.account.models import EmailAddress

from postorius.models import (
    MailmanApiError, MailmanListManager, MailmanUserManager)
from postorius.tests.utils import ViewTestCase


class ModelTest(ViewTestCase):
    """Tests for the list index page."""

    def setUp(self):
        super(ModelTest, self).setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.foo_list = self.domain.create_list('foo')

    @override_settings(AUTOCREATE_MAILMAN_USER=False)
    def test_mailman_user_not_created_when_flag_is_off(self):
        user = User.objects.create_user(
            'testuser', 'test@example.com', 'testpass')
        EmailAddress.objects.create(
            user=user, email=user.email, verified=True)
        with self.assertRaises(HTTPError):
            self.mm_client.get_user('test@example.com')

    @override_settings(AUTOCREATE_MAILMAN_USER=True)
    def test_mailman_user_created_when_flag_is_on(self):
        user = User.objects.create_user(
            'testuser', 'test@example.com', 'testpass')
        EmailAddress.objects.create(
            user=user, email=user.email, verified=True)
        mm_user = self.mm_client.get_user('test@example.com')
        self.assertEqual(str(mm_user.addresses[0]), 'test@example.com')

    @override_settings(AUTOCREATE_MAILMAN_USER=True)
    @patch('postorius.models.MailmanUser')
    def test_core_not_reachable(self, mock_model):
        # Fail Gracefully if the Core is not reachable when
        # creating the user account.
        mock_model.objects.create_from_django.side_effect = MailmanApiError
        # User creation should succeed without any error, even though there
        # is no MailmanUser. However, this error should be logged.
        with patch('postorius.models.logger') as log:
            User.objects.create_user('testuser, test@example.com', 'testpass')
            errmsg = 'Mailman Core API is not reachable.'
            log.error.assert_called_with(errmsg)
        # There should be no user in Mailman with this address.
        with self.assertRaises(HTTPError):
            self.mm_client.get_user('test@example.com')


class TestMailmanListManager(ViewTestCase):

    def setUp(self):
        super().setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.domain2 = self.mm_client.create_domain('most-desirable.org')
        self.foo_list = self.domain.create_list('foo')
        self.bar_list = self.domain.create_list('bar')
        self.baz_list = self.domain2.create_list('baz')
        self.list_manager = MailmanListManager()

    def test_get_all_mailinglists(self):
        lists = self.list_manager.all()
        # This should return all the 2 mailing lists that we have.
        self.assertEqual(len(lists), 3)
        self.assertEqual(
            [x.fqdn_listname for x in lists],
            ['bar@example.com', 'baz@most-desirable.org', 'foo@example.com'])

    def test_get_by_mail_host(self):
        lists = self.list_manager.by_mail_host('example.com')
        self.assertEqual(len(lists), 2)
        self.assertEqual(
            [x.fqdn_listname for x in lists],
            ['bar@example.com', 'foo@example.com'])

    def test_get_single_mailinglist(self):
        mlist = self.list_manager.get('baz@most-desirable.org')
        self.assertIsNotNone(mlist)
        self.assertEqual(str(mlist), str(self.baz_list))


class TestMailmanUserManager(ViewTestCase):

    @override_settings(AUTOCREATE_MAILMAN_USER=False)
    def setUp(self):
        super().setUp()
        self.user_manager = MailmanUserManager()
        self.bob = User.objects.create(
            email='bob@example.com', username='bob', first_name="Bob")
        self.alice = User.objects.create(
            email='alice@example.com', username='alice', first_name='Alice')

    def test_create_from_django_works(self):
        mm_user = self.user_manager.create_from_django(self.bob)
        self.assertIsNotNone(mm_user)
        self.assertEqual(len(mm_user.addresses), 1)

    def test_create_from_django_sets_all_attributes(self):
        mm_user = self.user_manager.create_from_django(self.bob)
        self.assertIsNotNone(mm_user)
        self.assertEqual(mm_user.display_name, 'Bob')
        self.assertEqual([str(x) for x in list(mm_user.addresses)],
                         ['bob@example.com'])

    def test_get_or_create_from_django(self):
        self.user_manager.create_from_django = MagicMock(name='create')
        muser = self.user_manager.get_or_create_from_django(self.bob)
        self.assertIsNotNone(muser)
        self.user_manager.create_from_django.assert_called_once()
        # This was a non-existent user so it was created for us. Now, let's try
        # with an existing user.
        user = self.user_manager.create(email=self.bob.email, password=None)
        self.assertIsNotNone(user)
        # Now we reset the mock and see that create_from_django isn't called
        # anymore.
        self.user_manager.create_from_django.reset_mock()
        muser = self.user_manager.get_or_create_from_django(self.bob)
        self.user_manager.create_from_django.assert_not_called()



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2021 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.urls import reverse

from allauth.account.models import EmailAddress
from django_mailman3.models import MailDomain

from postorius.tests.utils import ViewTestCase


class TestDomainEdit(ViewTestCase):

    def setUp(self):
        super().setUp()
        self.domain = self.mm_client.create_domain('example.com')
        self.user = User.objects.create_user('user', 'user@example.com', 'pwd')
        self.superuser = User.objects.create_superuser(
            'su', 'su@example.com', 'pwd')
        for user in (self.user, self.superuser):
            EmailAddress.objects.create(
                user=user, email=user.email, verified=True)
        MailDomain.objects.create(
            site=Site.objects.get_current(), mail_domain='example.com')
        self.url = reverse('domain_edit', args=['example.com'])

    def test_permission_denied(self):
        self.client.login(username='user', password='pwd')
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 403)

    def test_edit_non_existent_domain(self):
        self.client.login(username='su', password='pwd')
        response = self.client.get(reverse('domain_edit', args=['random.com']))
        self.assertEqual(response.status_code, 404)
        self.assertEqual(response.context['exception'],
                         'Domain does not exist')

    def test_edit_domain_works(self):
        self.client.login(username='su', password='pwd')
        response = self.client.post(
            self.url,
            dict(mail_host='example.com',
                 description='This is a new list description',
                 site=Site.objects.get_current().id))
        self.assertEqual(response.status_code, 302)
        self.assertEqual(self.domain.description,
                         'This is a new list description')

    def test_change_mail_host(self):
        new_site = Site.objects.create(domain='most-desirable.org')
        self.client.login(username='su', password='pwd')
        response = self.client.post(self.url,
                                    dict(mail_host='example.com',
                                         site=new_site.id))
        self.assertEqual(response.status_code, 302)
        mdomain = MailDomain.objects.get(mail_domain=self.domain.mail_host)
        self.assertEqual(mdomain.site, new_site)
